@@ -1,5 +1,5 @@
 // ****************************************************************************
-// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>  
+// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>
 // This file is part of the hk_classes library.
 // This file may be distributed and/or modified under the terms of the
 // GNU Library Public License version 2 as published by the Free Software
@@ -8,7 +8,7 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 // ****************************************************************************
-//$Revision: 1.59 $
+//$Revision: 1.63 $
 #include "hk_column.h"
 #include "hk_datasource.h"
 #include "hk_dsdatavisible.h"
@@ -529,7 +529,7 @@ hk_string hk_column::bool_falsevalue(void)
 unsigned int hk_column::find(unsigned int from,unsigned int to,const hk_string& searchtext,bool wholephrase,bool casesensitive,bool backwards)
 {
 #ifdef HK_DEBUG
-    hkdebug("hk_column::find(unsigned int from,unsigned int to,const hk_string& searchtext");
+    hkdebug("hk_column::find(unsigned int from,unsigned int to,const hk_string& searchtext",searchtext);
 #endif
     unsigned int p_from, p_to, p_buffer;
     if (p_datasource->max_rows()==0)return 1;
@@ -591,7 +591,7 @@ bool hk_column::is_findstring(unsigned int pos,const hk_string& searchtext,bool
 unsigned int hk_column::find(const hk_string& searchtext,bool wholephrase,bool casesensitive,bool backwards)
 {
 #ifdef HK_DEBUG
-    hkdebug("hk_column::find(const hk_string& searchtext");
+    hkdebug("hk_column::find(const hk_string& searchtext)",searchtext);
 #endif
     return find(0,p_datasource->max_rows()-1,searchtext,wholephrase,casesensitive,backwards);
 }
@@ -630,14 +630,13 @@ bool hk_column::save_to_file(const hk_string& filename)
     hkdebug("hk_column::save_to_file");
 #endif
 
-    FILE* out=fopen(filename.c_str(),"w");
+   if (columntype()==binarycolumn)
+   {
     const struct_raw_data* res=asbinary();
     if (res==NULL)return false;
-
-    int r=fwrite(res->data,res->length,1,out);
-    fclose(out);
-    if (r<(signed)res->length)return false;
-    return true;
+    return save_file(filename,res);
+   }
+   else return save_textfile(filename,asstring());
 }
 
 
@@ -774,6 +773,7 @@ void hk_column::load_columndefinition(const hk_string& definition)
     hk_string buffer;
 
     get_tagvalue(definition,"COLUMNNAME",p_columnname);
+    p_columnname=p_datasource->systemcolumnname(p_columnname);
     get_tagvalue(definition,"COLUMNTYPE",buffer);
 
     if ( buffer=="TEXTFIELD")f=textcolumn ;
@@ -990,7 +990,7 @@ long int hk_column::count(unsigned int from,unsigned int to)
  {
    if (!is_nullvalue_at(c))
      ++result;
-   ++c; 
+   ++c;
  }
  return result;
 }
@@ -1005,9 +1005,9 @@ double hk_column::sum(unsigned int from,unsigned int to)
  {
    if (!is_nullvalue_at(c))
      result+=asdouble_at(c);
-   ++c; 
+   ++c;
  }
- 
+
  return result;
 
 }
