@@ -1,5 +1,5 @@
 // ****************************************************************************
-// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>  
+// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>
 // This file is part of the hk_classes library.
 // This file may be distributed and/or modified under the terms of the
 // GNU Library Public License version 2 as published by the Free Software
@@ -8,7 +8,7 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 // ****************************************************************************
-//$Revision: 1.64 $
+//$Revision: 1.73 $
 #include "hk_connection.h"
 #include "hk_database.h"
 #include "hk_datasource.h"
@@ -150,7 +150,7 @@ bool  hk_connection::connect(enum_interaction c)
     if  (p_connected)return true;
     bool cancel=false;
     int logincounter=0;
-    while ((!is_connected())&&(!cancel)&&logincounter<3)
+    while ((!is_connected())&&(!cancel)&&logincounter<4)
     {
         if (!cancel) driver_specific_connect();
         if (!is_connected())
@@ -158,7 +158,8 @@ bool  hk_connection::connect(enum_interaction c)
 	      if ( c==noninteractive) cancel=true;
               else
 	      {
-	        show_warningmessage(hk_translate("Servermessage: ")+last_servermessage());
+	        if (logincounter>0)
+                  show_warningmessage(hk_translate("Servermessage: ")+last_servermessage());
 		cancel=!show_passworddialog();
 		++logincounter;
 	      }
@@ -191,7 +192,7 @@ bool hk_connection::show_passworddialog(void)
     hkdebug("hk_connection::show_passworddialog");
 #endif
     struct_connectionfields fields;
-    if (server_needs(NEEDS_NOTHING)) return true;
+    if (!server_needs(NEEDS_LOGIN)) return true;
     if (p_passworddialog!=NULL)
     {
         fields.host=host();
@@ -278,7 +279,7 @@ bool hk_connection::show_passworddialog(void)
             cerr <<hk_translate("Emulate Boolean: ");
             cerr <<hk_translate("n=No            all other keys = yes")<<endl;
             hk_string i;
-	    cin >>i;    
+	    cin >>i;
 		if (i=="n"||i=="N") fields.emulateboolean=false;
 				else fields.emulateboolean=true;
         }
@@ -295,7 +296,7 @@ bool hk_connection::show_passworddialog(void)
             p_private->p_tcp_port=atoi(fields.port.c_str());
         if (server_needs(NEEDS_BOOLEANEMULATION))
                 set_booleanemulation(fields.emulateboolean);
-	    
+
         return true;                              //not cancel
 
     }
@@ -333,11 +334,11 @@ hk_database* hk_connection::new_database(const hk_string& name)
     hkdebug("hk_connection::new_database");
 #endif
     if (name.size()>0&& !database_exists(name)) return NULL;
-     
-       
-     
+
+
+
     if (p_database!=NULL)
-    {  
+    {
         p_database->inform_datasources_before_closing();
         delete p_database;
     }
@@ -380,11 +381,23 @@ bool hk_connection::create_database(const hk_string& dbase)
 #ifdef HK_DEBUG
     hkdebug("hk_connection::create_database");
 #endif
+
+
+    if (!server_supports(SUPPORTS_NEW_DATABASE))
+      {
+       show_warningmessage(hk_translate("The driver does not support creating new databases!"));
+       return false;
+      }
     if (p_database==NULL) new_database();
-    if (p_database==NULL||!is_connected()) return false;
+    if (p_database==NULL||!is_connected())
+    {
+      show_warningmessage(hk_translate("Not connected to the server"));
+      return false;
+    }
     hk_actionquery* q=p_database->new_actionquery();
     if (!q)
     	{
+               show_warningmessage(hk_translate("Bug: Actionquery could not be created"));
 		return false;
 	}
     hk_string sql="CREATE DATABASE ";
@@ -519,7 +532,7 @@ void hk_connection::load_configuration(void)
     hkdebug("hk_connection::load_configuration");
 #endif
     hk_string n=p_private->p_classespath+"/driver.conf";
-    
+
     hk_string result;
     ifstream ifs(n.c_str(),ios::in);
     if (ifs)
@@ -653,9 +666,9 @@ bool    hk_connection::database_exists(const hk_string& databasename)
       ifstream ifs(databasename.c_str(),ios::in);
       if (ifs )
          return true;
-    }   
-  }          
-	  
+    }
+  }
+
 
 
     vector<hk_string>* liste=dblist();
@@ -676,7 +689,11 @@ bool hk_connection::copy_database(hk_database* fromdatabase,bool schema_and_data
 {
     bool cancel=false;
     if (!fromdatabase)return false;
-    bool dbexists=database_exists(fromdatabase->name());
+   cerr <<check_capabilities(fromdatabase->connection(),this)<<endl;
+   hk_url url=fromdatabase->name();
+   hk_string targetname=url.filename();
+    bool dbexists=database_exists(targetname);
+
     if (dbexists && runtime_only())
      {
        return false;
@@ -699,16 +716,17 @@ bool hk_connection::copy_database(hk_database* fromdatabase,bool schema_and_data
 
     } else
     {
-        create_database(fromdatabase->name());
-        newdb->set_name(fromdatabase->name());
+        if (!create_database(targetname))return false;
+        newdb->set_name(targetname);
     }
-    vector<hk_string>::iterator it=fromdatabase->tablelist()->begin();
 
 //now copy all tables
 
+    vector<hk_string>::iterator it=fromdatabase->tablelist()->begin();
     while (it!=fromdatabase->tablelist()->end())
     {
         hk_datasource* fds= fromdatabase->new_table((*it));
+	if (!fds) return false;
         cancel=!newdb->copy_table(fds,schema_and_data,true,true,progressdialog);
         delete fds;
         it++;
@@ -727,9 +745,38 @@ bool hk_connection::copy_database(hk_database* fromdatabase,bool schema_and_data
         it++;
     }
     }
-    
-    
-    
+
+
+//now copy referential integrity
+    if (fromdatabase->connection()->server_supports(SUPPORTS_REFERENTIALINTEGRITY)
+       &&server_supports(SUPPORTS_REFERENTIALINTEGRITY)
+    )
+    {
+    it=fromdatabase->tablelist()->begin();
+    while (it!=fromdatabase->tablelist()->end())
+    {
+        if (newdb->table_exists((*it)))
+        {
+          hk_datasource* fds=fromdatabase->new_table((*it));
+          hk_datasource* tds=newdb->new_table((*it));
+          if (fds &&tds)
+          {
+          list <referentialclass>* l=fds->referenceslist();
+          list <referentialclass>::iterator rit=l->begin();
+          while (rit!=l->end())
+          {
+            tds->add_reference(*rit);
+            ++rit;
+          }
+          delete fds;
+          delete tds;
+          }
+        }
+        it++;
+    }
+    }
+
+
     if (cpy_lcl_files )
     {                                             //now copy all queries,forms and reports
 
@@ -760,7 +807,7 @@ bool hk_connection::copy_local_files(hk_database* fromdb,hk_database* todb,objec
         default: return false;
     }
     if (!namelist) return false;
-    
+
     vector<hk_string>::iterator it=namelist->begin();
     int r=1;
     while (it!=namelist->end())
@@ -791,7 +838,9 @@ void hk_connection::delete_databasedirectory(const hk_string& db)
 #ifdef HK_DEBUG
     hkdebug("hk_database::delete_databasedirectory");
 #endif
-    hk_string dir=databasepath()+"/"+db;
+cerr <<"delete_databasedirectory "<<db<<endl;
+    hk_url url=db;
+    hk_string dir=(url.directory().size()>0?db:databasepath()+"/"+db);
     delete_directory(dir+"/output");
 
     delete_directory(dir);
@@ -804,7 +853,7 @@ void hk_connection::delete_directory(const hk_string& d)
 #ifdef HK_DEBUG
     hkdebug("hk_database::delete_directory");
 #endif
-
+cerr <<"delete directory: d="<<d<<endl;
     DIR *dp;
     struct dirent *entry;
     hk_string filename;
@@ -833,7 +882,7 @@ unsigned int    hk_connection::default_tcp_port(void) const
 {
     return 0;
 }
-                             
+
 
 void hk_connection::set_booleanemulation(bool emulate)
 {
@@ -845,7 +894,7 @@ bool hk_connection::booleanemulation(void) const
 {
   return p_private->p_booleanemulation;
 }
-  
+
 bool hk_connection::rename_database(const hk_string& originalname, const hk_string& new_name)
 {
 return false;
@@ -870,3 +919,59 @@ hk_string hk_connection::defaultdatabase(void) const
 
 
 
+hk_string hk_connection::check_capabilities(hk_connection* from,hk_connection* to)
+{
+ hk_string result;
+ if ((!from||!to)||(from==to)) return result;
+ if (!to->server_supports(SUPPORTS_NEW_DATABASE)) result+="* "+hk_translate("Can't create a new target database")+"\n";
+ if (!to->server_supports(SUPPORTS_NEW_TABLE)) result+="* "+hk_translate("Can't create new tables")+"\n";
+ if (to->maxfieldnamesize()<from->maxfieldnamesize())result+="* "+replace_all("%1",hk_translate("Field names will be truncated to %1 characters"),ulongint2string(to->maxfieldnamesize()))+"\n";
+ if (from->server_supports(SUPPORTS_PROPRIETARYCOLUMN)) result+="* "+hk_translate("Source database may contain proprietary field types")+"\n";
+ if (!internal_copy_check(from,to,SUPPORTS_NONALPHANUM_FIELDNAMES)) result+="* "+hk_translate("Field names with other characters than [A-Za-z0-9] are not allowed")+"\n";
+ if (!internal_copy_check(from,to,SUPPORTS_NONASCII_FIELDNAMES)) result+="* "+hk_translate("Field names with non-ASCII characters are not supported")+"\n";
+ if (!internal_copy_check(from,to,SUPPORTS_SPACE_FIELDNAMES)) result+="* "+hk_translate("Field names with space characters are not supported")+"\n";
+
+
+ if (!internal_copy_check(from,to,SUPPORTS_TEXTCOLUMN)) result+="* "+hk_translate("No 'char' field type")+"\n";
+ if (!internal_copy_check(from,to,SUPPORTS_INTEGERCOLUMN)) result+="* "+hk_translate("No 'integer' field type")+"\n";
+ if (!internal_copy_check(from,to,SUPPORTS_SMALLINTEGERCOLUMN)) result+="* "+hk_translate("No 'smallinteger' field type")+"\n";
+ if (!internal_copy_check(from,to,SUPPORTS_FLOATINGCOLUMN)) result+="* "+hk_translate("No 'floating' field type")+"\n";
+ if (!internal_copy_check(from,to,SUPPORTS_SMALLFLOATINGCOLUMN)) result+="* "+hk_translate("No 'smallfloating' field type")+"\n";
+
+ if (!internal_copy_check(from,to,SUPPORTS_AUTOINCCOLUMN)) result+="* "+hk_translate("No 'autoincrement' field type")+"\n";
+ if (!internal_copy_check(from,to,SUPPORTS_BOOLCOLUMN)) result+="* "+hk_translate("No 'boolean' field type")+"\n";
+ if (!internal_copy_check(from,to,SUPPORTS_DATECOLUMN)) result+="* "+hk_translate("No 'date' field type")+"\n";
+ if (!internal_copy_check(from,to,SUPPORTS_TIMECOLUMN)) result+="* "+hk_translate("No 'time' field type")+"\n";
+ if (!internal_copy_check(from,to,SUPPORTS_DATETIMECOLUMN)) result+="* "+hk_translate("No 'datetime' field type")+"\n";
+ if (!internal_copy_check(from,to,SUPPORTS_BINARYCOLUMN)) result+="* "+hk_translate("No 'binary' field type")+"\n";
+ if (!internal_copy_check(from,to,SUPPORTS_MEMOCOLUMN)) result+="* "+hk_translate("No 'memo' field type")+"\n";
+ if (!internal_copy_check(from,to,SUPPORTS_TIMESTAMPCOLUMN)) result+="* "+hk_translate("No 'timestamp' field type")+"\n";
+ if (!internal_copy_check(from,to,SUPPORTS_SQL)) result+="* "+hk_translate("Target does not support SQL")+"\n";
+if (!internal_copy_check(from,to,SUPPORTS_SQL_GROUP_BY)) result+="* "+hk_translate("SQL does not support 'GROUP BY' statement")+"\n";
+ if (!internal_copy_check(from,to,SUPPORTS_SQL_ORDER_BY)) result+="* "+hk_translate("SQL does not support 'ORDER BY' statement")+"\n";
+ if (!internal_copy_check(from,to,SUPPORTS_SQL_HAVING)) result+="* "+hk_translate("SQL does not support 'HAVING' statement")+"\n";
+ if (!internal_copy_check(from,to,SUPPORTS_SQL_WHERE)) result+="* "+hk_translate("SQL does not support 'WHERE' statement")+"\n";
+ if (!internal_copy_check(from,to,SUPPORTS_SQL_ALIAS)) result+="* "+hk_translate("SQL does not support 'ALIAS' statement")+"\n";
+
+ if (!internal_copy_check(from,to,SUPPORTS_REFERENTIALINTEGRITY)) result+="* "+hk_translate("Referential integrity not supported")+"\n";
+ if (!internal_copy_check(from,to,SUPPORTS_CREATE_INDEX)) result+="* "+hk_translate("Indices are not supported")+"\n";
+ if (!internal_copy_check(from,to,SUPPORTS_VIEWS)
+    ||!internal_copy_check(from,to,SUPPORTS_NEW_VIEW)
+    ) result+="* "+hk_translate("Views are not supported")+"\n";
+
+ if (result.size()>0)
+ result=replace_all("%2",replace_all("%1",hk_translate("Copying data from '%1' to '%2':\n"),from->drivername()),to->drivername())+result;
+
+
+ return result;
+
+}
+
+bool hk_connection::internal_copy_check(hk_connection* from, hk_connection* to,support_enum feature)
+{
+  if (!from||!to) return true;
+  if (from->server_supports(feature)&&!to->server_supports(feature)) return false;
+return true;
+}
+
+
