@@ -1,5 +1,5 @@
 // ****************************************************************************
-// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>  
+// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>
 // This file is part of the hk_classes library.
 // This file may be distributed and/or modified under the terms of the
 // GNU Library Public License version 2 as published by the Free Software
@@ -8,7 +8,7 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 // ****************************************************************************
-//$Revision: 1.95 $
+//$Revision: 1.114 $
 #include "hk_database.h"
 #include "hk_data.h"
 #include "hk_dbvisible.h"
@@ -26,11 +26,6 @@
 #include "hk_url.h"
 
 #include <sys/stat.h>
-#ifdef HAVE_SSTREAM
-#include <sstream>
-#else
-#include <strstream.h>
-#endif
 #include <dirent.h>
 #include <unistd.h>
 
@@ -47,7 +42,8 @@ public:
 hk_databaseprivate(void)
 			{
 			  p_parent=NULL;
-			  for (int f=ft_table;f<=ft_report;++f)
+			  p_automatic_data_update=hk_class::default_automatic_data_update();
+			  for (int f=ft_table;f<=ft_referentialintegrity;++f)
 			  {
 			    p_storagemode[f]=hk_database::local;
 			    p_loadmode[f]=hk_database::local;
@@ -61,8 +57,10 @@ hk_databaseprivate(void)
         list<hk_dbvisible*> p_visibles;
         list<hk_presentation*> p_presentations;
 	hk_class* p_parent;// will be used as widget parent in e.g.  ::new_formvisible;
-	hk_database::enum_storagemode p_storagemode[ft_report+1];
-	hk_database::enum_storagemode p_loadmode[ft_report+1];
+	hk_database::enum_storagemode p_storagemode[ft_referentialintegrity+1];
+	hk_database::enum_storagemode p_loadmode[ft_referentialintegrity+1];
+	hk_string p_charset;
+	bool p_automatic_data_update;
 };
 
 
@@ -131,7 +129,7 @@ vector<hk_string>* hk_database::tablelist(bool with_systemtable)
       if(it!=p_tablelist.end()) p_tablelist.erase(it);
     }
     return &p_tablelist;
-    
+
 }
 
 
@@ -139,7 +137,7 @@ vector<hk_string>* hk_database::viewlist()
 {
     driver_specific_viewlist();
     return &p_viewlist;
-    
+
 }
 
 vector<hk_string>* hk_database::tableviewlist(bool with_systemtable)
@@ -150,10 +148,10 @@ vector<hk_string>* hk_database::tableviewlist(bool with_systemtable)
    vector<hk_string>::iterator it=p_tablelist.begin();
    p_tableviewlist.insert(p_tableviewlist.end(),p_tablelist.begin(),p_tablelist.end());
    p_tableviewlist.insert(p_tableviewlist.end(),p_viewlist.begin(),p_viewlist.end());
-   sort(p_tableviewlist.begin(),p_tableviewlist.end()); 
-    
+   sort(p_tableviewlist.begin(),p_tableviewlist.end());
+
     return &p_tableviewlist;
-    
+
 }
 
 
@@ -173,7 +171,6 @@ bool hk_database::table_exists(const hk_string& tablename)
 
         it++;
     }
-
     return res;
 }
 
@@ -326,7 +323,7 @@ hk_actionquery* hk_database::new_actionquery(void)
 #ifdef HK_DEBUG
     hkdebug("hk_database::new_actionquery");
 #endif
-if (!p_connection->database_exists(p_private->p_dbname)) return NULL;
+//if (!p_connection->database_exists(p_private->p_dbname)) return NULL;
     hk_actionquery* tb=driver_specific_new_actionquery();
     if (tb!=NULL) p_private->p_hkdsourcelist.insert(p_private->p_hkdsourcelist.end(),tb);
     return tb;
@@ -362,7 +359,7 @@ bool hk_database::select_db(const hk_string& newname)
         {
             if (p_connection->server_supports(hk_connection::SUPPORTS_LOCAL_FILEFORMAT)&&
 	    p_url.is_valid() && p_url.directory().size() >0)
-	    { 
+	    {
 	      //cerr <<"directory=#"<<p_url.directory()<<"#"<<endl;
 	      //cerr <<"local file name"<<endl;
 	    p_private->p_databasepath=p_connection->databasepath();
@@ -392,7 +389,7 @@ bool hk_database::select_db(const hk_string& newname)
 	    }
 	    else
 	    {
-	     for (int f=ft_query;f<=ft_report;++f)
+	     for (int f=ft_query;f<=ft_referentialintegrity;++f)
 	      {
 		    p_private->p_storagemode[f]=hk_database::local;
 		    p_private->p_loadmode[f]=hk_database::local;
@@ -404,8 +401,8 @@ bool hk_database::select_db(const hk_string& newname)
         else
 	{
           p_private->p_dbname=oldname;
-          show_warningmessage(hk_translate("No such Database!"));
-	}     
+          show_warningmessage(replace_all("%1",hk_translate("No such Database '%1'!"),newname));
+	}
         return erg;
     }
     else
@@ -619,17 +616,27 @@ void hk_database::inform_datasources_before_closing(void)
 
 void hk_database::save(const hk_string& statement,const hk_string& name,filetype type,bool ask_before_overwrite,bool ask_for_new_name)
 {
+  save(st_overwrite,statement,name,type,ask_before_overwrite,ask_for_new_name);
+}
+void hk_database::save(enum_storage storage,const hk_string& statement,const hk_string& name,filetype type,bool ask_before_overwrite,bool ask_for_new_name)
+{
+
 #ifdef HK_DEBUG
     hkdebug("hk_database::save");
 #endif
-  
-return (p_private->p_storagemode[type]==central?save_central(statement,name,type,ask_before_overwrite,ask_for_new_name):save_local(statement,name,type,ask_before_overwrite,ask_for_new_name));
+
+return (p_private->p_storagemode[type]==central?save_central(storage,statement,name,type,ask_before_overwrite,ask_for_new_name):save_local(storage,statement,name,type,ask_before_overwrite,ask_for_new_name));
 }
 
 
 
 void hk_database::save_local(const hk_string& statement,const hk_string& name,filetype type,bool ask_before_overwrite,bool ask_for_new_name)
 {
+save_local( st_overwrite,statement,name,type,ask_before_overwrite,ask_for_new_name);
+}
+
+void hk_database::save_local(enum_storage storage, const hk_string& statement,const hk_string& name,filetype type,bool ask_before_overwrite,bool ask_for_new_name)
+{
 #ifdef HK_DEBUG
     hkdebug("hk_database::save_local");
 #endif
@@ -639,23 +646,29 @@ void hk_database::save_local(const hk_string& statement,const hk_string& name,fi
     s->close();
     delete s;
     return ;
-    
+
 
 }
 
 
 void hk_database::save_central(const hk_string& statement,const hk_string& name,filetype type,bool ask_before_overwrite,bool ask_for_new_name)
 {
+save_central(st_overwrite,statement,name,type,ask_before_overwrite,ask_for_new_name);
+}
+
+
+void hk_database::save_central(enum_storage storage,const hk_string& statement,const hk_string& name,filetype type,bool ask_before_overwrite,bool ask_for_new_name)
+{
 #ifdef HK_DEBUG
     hkdebug("hk_database::save_central");
 #endif
 hk_datasource* ds=new_table("HKCLASSES");
- if (!ds) 
+ if (!ds)
    {
    show_warningmessage(hk_translate("Error: hk_database::save_central could not get a new table"));
    return;
    }
- hk_string filter="\"type\"="+longint2string(type); 
+ hk_string filter="\"type\"="+longint2string(type);
  ds->set_filter(filter);
  ds->enable();
  hk_column* n=ds->column_by_name("name");
@@ -663,13 +676,13 @@ hk_datasource* ds=new_table("HKCLASSES");
  hk_column* t=ds->column_by_name("type");
  hk_column* u=ds->column_by_name("user");
  hk_column* p=ds->column_by_name("update");
- if (!n ||!v||!t) 
+ if (!n ||!v||!t)
    {
     show_warningmessage(hk_translate("Error: hk_database::save_central could not find system columns!"));
     delete ds;
     return;
    }
-  unsigned int row=n->find(name,true,true);    
+  unsigned int row=n->find(name,true,true);
   if (row > ds->max_rows())
   { // not existing => append object
     ds->setmode_insertrow();
@@ -680,12 +693,8 @@ hk_datasource* ds=new_table("HKCLASSES");
   {  // replace existing object
     if (ask_before_overwrite )
     {
-    
-    hk_string dialog;
-    if (type==ft_query)dialog=hk_translate("Query already exists. Overwrite it?");
-    else if (type==ft_form)dialog=hk_translate("Form already exists. Overwrite it?");
-    else if (type==ft_report)dialog=hk_translate("Report already exists. Overwrite it?");
-    else dialog=replace_all("%1",hk_translate("File '%1'already exists. Overwrite it?"),name);
+
+    hk_string dialog=savemessage(type,storage,name);
      if(runtime_only() || !show_yesnodialog(dialog,true))
             {
               // no, so don't overwrite it
@@ -696,7 +705,7 @@ hk_datasource* ds=new_table("HKCLASSES");
                     else if (type==ft_report)dialog=hk_translate("Enter new report name:");
                     else dialog=hk_translate("Enter new name:");
                     hk_string res=show_stringvaluedialog(dialog);
-                    if (res.size()==0) 
+                    if (res.size()==0)
 		       {
 		         delete ds;
 		         return ;
@@ -710,18 +719,18 @@ hk_datasource* ds=new_table("HKCLASSES");
 		{
 		    delete ds;
                     return ;
-		}    
+		}
             }
         }
-    
-    
-    
-    
+
+
+
+
     ds->goto_row(row);
   }
   v->set_asstring(statement);
   if (u) u->set_asstring(p_connection->user());
-  if (p) 
+  if (p)
     {
      hk_datetime date;
      date.set_now();
@@ -741,19 +750,41 @@ ofstream* hk_database::savestream(const hk_string&name,filetype type,bool ask_be
     return savestream(name,type,ask_before_overwrite,true,false);
 }
 
+hk_string hk_database::savemessage(filetype type,enum_storage storage,hk_string name)
+{
+ hk_string dialog;
+ if (storage==st_overwrite)
+ {
+    if (type==ft_query)dialog=replace_all("%1",hk_translate("Query '%1' already exists. Overwrite it?"),name);
+    else if (type==ft_form)dialog=replace_all("%1",hk_translate("Form '%1' already exists. Overwrite it?"),name);
+    else if (type==ft_report)dialog=replace_all("%1",hk_translate("Report '%1' already exists. Overwrite it?"),name);
+    else dialog=replace_all("%1",hk_translate("File '%1' already exists. Overwrite it?"),name);
+ }
+ else
+ {
+    if (type==ft_query)dialog=replace_all("%1",hk_translate("Query '%1' has changed. Store changes?"),name);
+    else if (type==ft_form)dialog=replace_all("%1",hk_translate("Form '%1' has changed. Store changes?"),name);
+    else if (type==ft_report)dialog=replace_all("%1",hk_translate("Report '%1' has changed. Store changes?"),name);
+    else dialog=replace_all("%1",hk_translate("File '%1' has changed. Store changes?"),name);
+ }
+ return dialog;
+}
 
 ofstream* hk_database::savestream(const hk_string&name,filetype type,bool ask_before_overwrite,bool with_header,bool ask_for_new_name)
+{
+  return savestream(st_overwrite,name,type,ask_before_overwrite,with_header,ask_for_new_name);
+}
+
+
+ofstream* hk_database::savestream(enum_storage storage,const hk_string&name,filetype type,bool ask_before_overwrite,bool with_header,bool ask_for_new_name)
 {
 #ifdef HK_DEBUG
     hkdebug("hk_database::savestream");
 #endif
     hk_string filename=p_private->p_databasepath+"/";
     filename+=name+fileendings(type);
-    hk_string dialog;
-    if (type==ft_query)dialog=hk_translate("Query already exists. Overwrite it?");
-    else if (type==ft_form)dialog=hk_translate("Form already exists. Overwrite it?");
-    else if (type==ft_report)dialog=hk_translate("Report already exists. Overwrite it?");
-    else dialog=replace_all("%1",hk_translate("File '%1'already exists. Overwrite it?"),name);
+    hk_string dialog=savemessage(type,storage,name);
+    
     if (ask_before_overwrite)
     {
         ifstream ifs(filename.c_str(),ios::in);
@@ -815,29 +846,20 @@ ofstream* hk_database::savestream(const hk_string&name,filetype type,bool ask_be
     return savestringstream(type,true);
 }
 
-#ifdef HAVE_SSTREAM
-    stringstream*  hk_database::savestringstream(filetype type,bool with_header)
+    STRINGSTREAM*  hk_database::savestringstream(filetype type,bool with_header)
 
-#else
-    strstream*  hk_database::savestringstream(filetype type,bool with_header)
-
-#endif
 
 {
 #ifdef HK_DEBUG
     hkdebug("hk_database::savestringstream");
 #endif
 
-#ifdef HAVE_SSTREAM
-    stringstream* stream= new stringstream();
-#else
-    strstream* stream= new strstream();
-#endif
+    STRINGSTREAM* stream= new STRINGSTREAM();
     if (stream!=NULL &&!stream->fail())
     {
         if (with_header) (*stream) <<"<?xml version=\"1.0\" encoding=\"UTF-8\" ?>"<<endl;
-    } 
-    
+    }
+
     return stream;
 
 }
@@ -861,7 +883,6 @@ hk_string hk_database::load(const hk_string& name,filetype type)
 #ifdef HK_DEBUG
     hkdebug("hk_database::load");
 #endif
-  
 return (p_private->p_loadmode[type]==central?load_central(name,type):load_local(name,type));
 }
 
@@ -894,27 +915,27 @@ hk_string hk_database::load_central(const hk_string& name,filetype type)
 #endif
 
 hk_datasource* ds=new_table("HKCLASSES");
- if (!ds) 
+ if (!ds)
    {
    show_warningmessage(hk_translate("Error: hk_database::load_central could not get a new table"));
    return "";
    }
- hk_string result;  
- hk_string filter="\"type\"="+longint2string(type); 
+ hk_string result;
+ hk_string filter="\"type\"="+longint2string(type);
  ds->set_filter(filter);
  ds->enable();
  hk_column* n=ds->column_by_name("name");
  hk_column* v=ds->column_by_name("value");
  hk_column* t=ds->column_by_name("type");
- if (!n ||!v||!t) 
+ if (!n ||!v||!t)
    {
     show_warningmessage(hk_translate("Error: hk_database::load_central could not find system columns!"));
     delete ds;
     return "";
    }
-  unsigned int row=n->find(name,true,true);    
+  unsigned int row=n->find(name,true,true);
   if (row > ds->max_rows())
-  { // not existing 
+  { // not existing
      /*show_warningmessage(replace_all("%1",hk_translate("Error: hk_database::load_central object '%1' not found"),name));*/
      //cerr <<"no file: "<<n<<endl;
      delete ds;
@@ -945,6 +966,7 @@ hk_string hk_database::fileendings(filetype e)
         case ft_form: return ".hk_form";
         case ft_report: return ".hk_report";
         case ft_view: return ".hk_view";
+        case ft_referentialintegrity: return ".hk_referentialintegrity";
     }
     return ".hk_unknown";
 }
@@ -1003,7 +1025,7 @@ vector<hk_string>* hk_database::central_filelist(filetype type)
  p_private->p_filelist.erase(p_private->p_filelist.begin(),p_private->p_filelist.end());
  if (!has_centralstoragetable() || !p_connection->is_connected()) return &p_private->p_filelist;
  hk_datasource* ds=new_resultquery();
- if (!ds) 
+ if (!ds)
    {
    show_warningmessage(hk_translate("Error: hk_database::central_filelist could not get a new resultquery"));
    }
@@ -1017,35 +1039,35 @@ vector<hk_string>* hk_database::central_filelist(filetype type)
    type 2 = query
    type 3 = form
    type 4 = report
- */  
+ */
         case ft_table:  t="1";break;
         case ft_query:  t="2";break;
         case ft_form:   t="3";break;
         case ft_report: t="4";break;
-        default : 
+        default :
 	   {
 	     show_warningmessage(hk_translate("Warning: hk_database::central_filelist, unknown filetype"));
 	     t="-1";
 	   }
    }
-   
+
    hk_string query="SELECT * FROM \"HKCLASSES\" WHERE \"type\"="+t+" ORDER BY \"name\"";
    ds->set_sql(query);
    ds->enable();
    unsigned k=0;
    hk_column* name=ds->column_by_name("name");
-   if (name) 
+   if (name)
    {
      while (k<ds->max_rows())
      {
-       p_private->p_filelist.insert(p_private->p_filelist.end(),name->asstring());  
+       p_private->p_filelist.insert(p_private->p_filelist.end(),name->asstring());
        ds->goto_next();++k;
      };
    }
-   else show_warningmessage("hk_database::central_filelist ERROR could not find column!"); 
-   
- }  
- 
+   else show_warningmessage("hk_database::central_filelist ERROR could not find column!");
+
+ }
+
  delete ds;
 return &p_private->p_filelist;
 }
@@ -1105,7 +1127,7 @@ bool  hk_database::delete_file(const hk_string& name,filetype type,enum_interact
     if(x==interactive&&!show_yesnodialog(message,true))
         return false;
 return (p_private->p_loadmode[type]==central?delete_centralfile(name,type,x):delete_localfile(name,type,x));
-   
+
 }
 
 
@@ -1116,27 +1138,27 @@ bool  hk_database::delete_centralfile(const hk_string& name,filetype type,enum_i
 #endif
 
 hk_datasource* ds=new_table("HKCLASSES");
- if (!ds) 
+ if (!ds)
    {
    show_warningmessage(hk_translate("Error: hk_database::delete_centralfile could not get a new table"));
    return false;
    }
- hk_string result;  
- hk_string filter="\"type\"="+longint2string(type); 
+ hk_string result;
+ hk_string filter="\"type\"="+longint2string(type);
  ds->set_filter(filter);
  ds->enable();
  hk_column* n=ds->column_by_name("name");
  hk_column* v=ds->column_by_name("value");
  hk_column* t=ds->column_by_name("type");
- if (!n ||!v||!t) 
+ if (!n ||!v||!t)
    {
     show_warningmessage(hk_translate("Error: hk_database::delete_centralfile could not find system columns!"));
     delete ds;
     return false;
    }
-  unsigned int row=n->find(name,true,true);    
+  unsigned int row=n->find(name,true,true);
   if (row > ds->max_rows())
-  { // not existing 
+  { // not existing
      show_warningmessage(replace_all("%1",hk_translate("Error: hk_database::delete_centralfile object '%1' not found"),name));
      delete ds;
      return false;
@@ -1184,7 +1206,7 @@ void hk_database::inform_datasources_filelist_changes(listtype type)
         it++;
         vi->filelist_changes(type);
     }
-    
+
     mark_visible_objects_as_not_handled();
     list<hk_dbvisible*>::iterator dbit=p_private->p_visibles.begin();
     while ( dbit!=p_private->p_visibles.end() )
@@ -1208,10 +1230,10 @@ hk_datasource*  hk_database::load_datasource(const hk_string& name,datasourcetyp
 #ifdef HK_DEBUG
     hkdebug("hk_database::load_datasource");
 #endif
-    
+
     switch (dt)
     {
-      case dt_query: 
+      case dt_query:
     {
         hk_datasource* d=NULL;
 	if (name.size()==0) return new_resultquery(p);
@@ -1229,7 +1251,7 @@ hk_datasource*  hk_database::load_datasource(const hk_string& name,datasourcetyp
         }
     default:
         return new_table(name,p);
-      
+
     }
 
 }
@@ -1398,7 +1420,7 @@ hk_report* hk_database::existing_report(const hk_string name)
 {
   hk_report*r =dynamic_cast<hk_report*>(existing_presentation(name,hk_presentation::report));
   return r;
-  
+
 }
 
 
@@ -1410,8 +1432,9 @@ bool hk_database::copy_table(hk_datasource* fromdatasource,bool schema_and_data,
 #ifdef HK_DEBUG
     hkdebug("hk_database::copy_table");
 #endif
+    if (!connection()->server_supports(hk_connection::SUPPORTS_NEW_TABLE)) return false;
     if (!fromdatasource)return false;
-    if (fromdatasource->columns()->size()==0)return false;
+    if (fromdatasource->columnnames()->size()==0)return false;
     bool tblexists=table_exists(fromdatasource->name());
     if ((tblexists&&runtime_only()) || (tblexists&&!ask&&!replacetable))return false;
     hk_datasource* newds=new_table();
@@ -1426,17 +1449,24 @@ bool hk_database::copy_table(hk_datasource* fromdatasource,bool schema_and_data,
             char c;
         while (stream.get(c))
             structure+=c;
-
     bool result=newds->load_datasourcedefinition(u2l(structure),replacetable);
     bool cancel=false;
     if (result&&schema_and_data)
     {
         newds->set_accessmode(hk_datasource::batchwrite);
         newds->enable();
-        if (fromdatasource->is_enabled())fromdatasource->disable();
-        hk_datasource::enum_accessmode access=fromdatasource->accessmode();
+        bool source_was_enabled=fromdatasource->is_enabled();
+        fromdatasource->set_blockvisiblesignals(true);
+        unsigned long pos=fromdatasource->row_position();
+        hk_datasource::enum_accessmode access=hk_datasource::batchread;
+        if (!source_was_enabled)
+        {
+        access=fromdatasource->accessmode();
         fromdatasource->set_accessmode(hk_datasource::batchread);
         fromdatasource->enable();
+        }
+        else
+        fromdatasource->goto_first();
         unsigned long r=0;
 	if (!newds->columns())
 	  {
@@ -1456,13 +1486,16 @@ bool hk_database::copy_table(hk_datasource* fromdatasource,bool schema_and_data,
             list<hk_column*>::iterator it=fromdatasource->columns()->begin();
             while (it!=fromdatasource->columns()->end())
             {
-                hk_column* c=newds->column_by_name((*it)->name(),fromdatasource->columnname_occurance(*it));
+                hk_column* c=newds->column_by_name(newds->systemcolumnname((*it)->name()),fromdatasource->columnname_occurance(*it));
+		//cerr <<"    add c:"<<c<<" name:"<<newds->systemcolumnname((*it)->name())<<endl;
                 if (c&&!(*it)->is_nullvalue())
                 {
 
                     switch(c->columntype())
                     {
-                        case hk_column::binarycolumn: c->set_asbinary((*it)->asbinary());break;
+                        case hk_column::binarycolumn: if (connection()->server_supports(hk_connection::SUPPORTS_BINARYCOLUMN))
+							c->set_asbinary((*it)->asbinary());
+							break;
                         case hk_column::boolcolumn: c->set_asbool((*it)->asbool());break;
                         default: if ((*it)->asstring().size()>0)
                         {
@@ -1476,17 +1509,29 @@ bool hk_database::copy_table(hk_datasource* fromdatasource,bool schema_and_data,
             }
 
             if (progressdialog) cancel=progressdialog(r,fromdatasource->max_rows(),replace_all("%TABLE%",hk_translate("Copying table %TABLE%"),name()));
-            newds->store_changed_data();
+            if (!newds->store_changed_data())
+	    {
+	    if(!show_yesnodialog(hk_translate("Continue loading data?"),true))
+	      {
+	       newds->set_ignore_changed_data();
+	       cancel=true;
+	      }
+	    }
             fromdatasource->goto_next();
             r++;
         }
         newds->driver_specific_after_copy_table();
         newds->disable();
-        fromdatasource->disable();
-        fromdatasource->set_accessmode(access);
+        fromdatasource->set_blockvisiblesignals(false);
+        if (!source_was_enabled)
+        {
+         fromdatasource->disable();
+         fromdatasource->set_accessmode(access);
+        }
+        else
+        fromdatasource->goto_row(pos);
         delete newds;
     }
-
     if (result)inform_datasources_filelist_changes(lt_table);
     if (cancel) return false;
     return result;
@@ -1502,8 +1547,15 @@ bool hk_database::copy_view(hk_datasource* fromdatasource,progress_dialogtype* p
 #ifdef HK_DEBUG
     hkdebug("hk_database::copy_view");
 #endif
+cerr <<"copy_view"<<endl;
     if (!fromdatasource)return false;
-    if (fromdatasource->columns()->size()==0)return false;
+    //if (fromdatasource->columns()->size()==0)return false;
+    if (!connection()->server_supports(hk_connection::SUPPORTS_NEW_VIEW))
+    {
+     show_warningmessage(hk_translate("Error copy_view: target database does not support views!"));
+     return false;
+    }
+cerr <<"vor viewexists"<<endl;
     bool viewexists=view_exists(newname.size()==0?fromdatasource->name():newname);
     if ((viewexists&&runtime_only()))return false;
     hk_string res;
@@ -1526,17 +1578,19 @@ bool hk_database::copy_view(hk_datasource* fromdatasource,progress_dialogtype* p
 	  res=show_stringvaluedialog(hk_translate("Please enter the viewname:"));
           if (res.size()==0) return false;
 	  return copy_view(fromdatasource,progressdialog,res);
-       } 
+       }
       }
       else
         res=newname.size()==0?fromdatasource->name():newname;
+cerr<<"vor new_view"<<endl;
     hk_datasource* newds=new_view();
+    if (!newds) return false;
     newds->set_name(res);
     newds->set_sql(fromdatasource->sql());
-    
+cerr<<"vor create_view_now"<<endl;
     bool result=newds->create_view_now();
     if (result)inform_datasources_filelist_changes(lt_view);
-    
+cerr <<"vor delete"<<endl;
     delete newds;
     return result;
 }
@@ -1698,30 +1752,30 @@ bool hk_database::rename_table(const hk_string& originalname, const hk_string& n
 bool hk_database::rename_file(const hk_string& originalname, const hk_string& new_name,filetype ftype,enum_interaction x)
 {
     if (ftype==ft_table) return rename_table(originalname,new_name,x);
-    
+
     if (storagemode(ftype)==central)
     {
       hk_datasource* ds=new_table("HKCLASSES");
-         if (!ds) 
+         if (!ds)
           {
            show_warningmessage(hk_translate("Error: hk_database::rename_file could not get a new table"));
            return false;
           }
-     hk_string filter="\"type\"="+longint2string(ftype); 
+     hk_string filter="\"type\"="+longint2string(ftype);
      ds->set_filter(filter);
      ds->enable();
      hk_column* n=ds->column_by_name("name");
      hk_column* v=ds->column_by_name("value");
      hk_column* t=ds->column_by_name("type");
-     if (!n ||!v||!t) 
+     if (!n ||!v||!t)
        {
         show_warningmessage(hk_translate("Error: hk_database::rename_file could not find system columns!"));
         delete ds;
         return false;
        }
-     unsigned int row=n->find(originalname,true,true);    
+     unsigned int row=n->find(originalname,true,true);
      if (row > ds->max_rows())
-     { 
+     {
         show_warningmessage(hk_translate("Error: hk_database::rename file: No such file!"));
         delete ds;
         return false;
@@ -1735,7 +1789,7 @@ bool hk_database::rename_file(const hk_string& originalname, const hk_string& ne
     delete ds;
     return true;
     } // Central storage END
-    
+
     hk_string n=p_private->p_databasepath+"/"+originalname+fileendings(ftype);
 
     ifstream ifs(n.c_str(),ios::in);
@@ -1767,6 +1821,7 @@ bool hk_database::rename_file(const hk_string& originalname, const hk_string& ne
 
 void hk_database::set_storagemode(filetype type,enum_storagemode load,enum_storagemode store)
 {
+   if (type>ft_report) return;
    bool hascentraltable =has_centralstoragetable();
       p_private->p_loadmode[type]=((hascentraltable && load==hk_database::central)?
                              hk_database::central:hk_database::local
@@ -1778,11 +1833,15 @@ void hk_database::set_storagemode(filetype type,enum_storagemode load,enum_stora
 
 hk_database::enum_storagemode hk_database::storagemode(filetype f)
 {
+  if (f==ft_view||f==ft_referentialintegrity) return hk_database::local;
+  //if (f==ft_referentialintegrity) return p_private->p_storagemode[ft_form];
   return p_private->p_storagemode[f];
 }
 
 hk_database::enum_storagemode hk_database::loadmode(filetype f)
 {
+  if (f==ft_view||f==ft_referentialintegrity) return hk_database::local;
+  //if (f==ft_referentialintegrity) return p_private->p_storagemode[ft_form];
   return p_private->p_loadmode[f];
 
 }
@@ -1791,43 +1850,48 @@ bool hk_database::create_centralstoragetable(void)
 {
   if (has_centralstoragetable()) return false;
   hk_datasource *ds=new_table();
-  if (!ds) 
+  if (!ds)
     {
        show_warningmessage(hk_translate("Error: hk_database::create_centralstoragetable could not get a new table"));
        return false;
     }
   ds->setmode_createtable();
   hk_column* col=ds->new_column();
+  if (!col) return false;
   col->set_name("type");
   col->set_primary(true);
   col->set_columntype(hk_column::smallintegercolumn);
   col->set_notnull(true);
-  
+
   col=ds->new_column();
+  if (!col) return false;
   col->set_name("name");
   col->set_primary(true);
   col->set_columntype(hk_column::textcolumn);
-  col->set_size(255);
+  col->set_size(190);// reduced for Firebird which Primary key is not allowed to be >253 chars (primary key is: 'name'+'type'
   col->set_notnull(true);
-  
+
   col=ds->new_column();
+  if (!col) return false;
   col->set_name("value");
   col->set_primary(false);
   col->set_columntype(hk_column::memocolumn);
   col->set_notnull(true);
-  
+
   col=ds->new_column();
+  if (!col) return false;
   col->set_name("user");
   col->set_size(50);
   col->set_primary(false);
   col->set_columntype(hk_column::textcolumn);
 
   col=ds->new_column();
+  if (!col) return false;
   col->set_name("update");
   col->set_primary(false);
   col->set_columntype(hk_column::datetimecolumn);
-  
-    
+
+
   ds->set_name("HKCLASSES");
   bool result = ds->create_table_now();
   delete ds;
@@ -1839,7 +1903,7 @@ bool hk_database::create_centralstoragetable(void)
 bool hk_database::has_centralstoragetable(void)
 {
  driver_specific_tablelist();
- 
+
   return (find(p_tablelist.begin(),p_tablelist.end(),"HKCLASSES")!=p_tablelist.end());
 }
 
@@ -1868,14 +1932,14 @@ void hk_database::load_configuration(void)
 
 
 void hk_database::save_configuration(void)
-{  
+{
     mkdir (database_path().c_str(),S_IRUSR|S_IWUSR|S_IXUSR);
     hk_string p_filename =database_path()+"/database.conf";
     ofstream* stream= new ofstream(p_filename.c_str());
     if (stream==NULL)
       return;
     savedata(*stream);
-    
+
     stream->close();
     delete stream;
     chmod(p_filename.c_str(),S_IRUSR|S_IWUSR);
@@ -1888,6 +1952,9 @@ void hk_database::savedata(ostream& s)
     start_mastertag(s,mt);
     for (int f=ft_query;f<=ft_report;++f)
         save_storage(s,(filetype)f);
+    set_tagvalue(s,"DATABASECHARSET",p_private->p_charset);
+    set_tagvalue(s,"AUTOMATIC_DATA_UPDATE",is_automatic_data_update());
+    
     end_mastertag(s,mt);
 }
 
@@ -1896,11 +1963,11 @@ void hk_database::save_storage(ostream& s,filetype f)
     hk_string savetxt,loadtxt;
     switch (f)
     {
-     case ft_table: 
+     case ft_table:
      			savetxt="STORETABLE";
 			loadtxt="LOADTABLE";
      			break;
-     case ft_query: 
+     case ft_query:
      			savetxt="STOREQUERY";
 			loadtxt="LOADQUERY";
      			break;
@@ -1916,7 +1983,11 @@ void hk_database::save_storage(ostream& s,filetype f)
      			savetxt="STOREVIEW";
 			loadtxt="LOADVIEW";
      			break;
-			
+     case ft_referentialintegrity:
+     			savetxt="STOREREFERENTIALINTEGRITY";
+			loadtxt="LOADREFERENTIALINTEGRITY";
+     			break;
+
     };
     hk_string c=(p_private->p_storagemode[f]==hk_database::local?"LOCAL":"CENTRAL");
     set_tagvalue(s,savetxt,c);
@@ -1927,9 +1998,11 @@ void hk_database::save_storage(ostream& s,filetype f)
 
 void hk_database::loaddata(const hk_string& definition)
 {
+    get_tagvalue(definition,"DATABASECHARSET",p_private->p_charset);
+    get_tagvalue(definition,"AUTOMATIC_DATA_UPDATE",p_private->p_automatic_data_update);
    if (!has_centralstoragetable())
    {
-    for (int f=ft_table;f<=ft_report;++f)
+    for (int f=ft_table;f<=ft_referentialintegrity;++f)
      {
        p_private->p_storagemode[f]=hk_database::local;
        p_private->p_loadmode[f]=hk_database::local;
@@ -1937,7 +2010,7 @@ void hk_database::loaddata(const hk_string& definition)
     return;
    }
     for (int f=ft_query;f<=ft_report;++f)
-    { 
+    {
     load_storage(definition,(filetype)f);
     }
 
@@ -1949,11 +2022,11 @@ void hk_database::load_storage(const hk_string& definition,filetype f)
     hk_string savetxt,loadtxt;
     switch (f)
     {
-     case ft_table: 
+     case ft_table:
      			savetxt="STORETABLE";
 			loadtxt="LOADTABLE";
      			break;
-     case ft_query: 
+     case ft_query:
      			savetxt="STOREQUERY";
 			loadtxt="LOADQUERY";
      			break;
@@ -1969,12 +2042,16 @@ void hk_database::load_storage(const hk_string& definition,filetype f)
      			savetxt="STOREVIEW";
 			loadtxt="LOADVIEW";
      			break;
-			
+     case ft_referentialintegrity:
+     			savetxt="STOREREFERENTIALINTEGRITY";
+			loadtxt="LOADREFERENTIALINTEGRITY";
+     			break;
+
     };
     hk_string savebuffer,loadbuffer;
     get_tagvalue(definition,savetxt,savebuffer);
     get_tagvalue(definition,loadtxt,loadbuffer);
-    
+
     set_storagemode(f,(loadbuffer=="CENTRAL"?central:local),(savebuffer=="CENTRAL"?central:local));
 
 }
@@ -1988,5 +2065,60 @@ void hk_database::disable()
  p_private->p_dbname="";
 }
 
+bool hk_database::store_connectionfile(const hk_url& url, bool store_password)
+{
+        ifstream ifs(url.url().c_str(),ios::in);
+        if (ifs )
+        {
+//file already exists, overwrite it?
+            if(runtime_only() || !show_yesnodialog(replace_all("%1",hk_translate("Overwrite '%1'?"),url.url()),true))
+            {
+              return false;
+            }
+        }
+
+    ofstream* stream= new ofstream(url.url().c_str());
+    if (stream!=NULL &&!stream->fail())
+	{
+        (*stream) <<"<?xml version=\"1.0\" encoding=\"UTF-8\" ?>"<<endl;
+ set_tagvalue(*stream,"CONNECTION",p_connection->drivername());
+ set_tagvalue(*stream,"DATABASE",name());
+ set_tagvalue(*stream,"USER",p_connection->user());
+ set_tagvalue(*stream,"PASSWORD",(store_password?p_connection->password():""));
+ set_tagvalue(*stream,"HOST",p_connection->host());
+ set_tagvalue(*stream,"TCPPORT",p_connection->tcp_port());
+ set_tagvalue(*stream,"BOOLEANEMULATION",p_connection->booleanemulation());
+
+	}
+    delete stream;
+   return true;
+}
+
+
+void hk_database::set_databasecharset(const hk_string& charset)
+{
+ p_private->p_charset=charset;
+}
+
+hk_string hk_database::databasecharset(void) const
+{
+return p_private->p_charset;
+}
+
+
+void hk_database::set_automatic_data_update(bool u)
+{
+#ifdef HK_DEBUG
+    hkdebug("hk_database::set_automatic_data_update");
+#endif
+    p_private->p_automatic_data_update=u;
+}
+
+
+bool hk_database::is_automatic_data_update(void)
+{
+    return p_private->p_automatic_data_update;
+}
+
 
 
