@@ -1,5 +1,5 @@
 // ****************************************************************************
-// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>  
+// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>
 // This file is part of the hk_classes library.
 // This file may be distributed and/or modified under the terms of the
 // GNU Library Public License version 2 as published by the Free Software
@@ -27,7 +27,7 @@ class hk_datasourceprivate;
 /**
  *
  *@short  represents a resultquery or a table of a database.
- *@version $Revision: 1.85 $
+ *@version $Revision: 1.93 $
  *@author Horst Knorr (hk_classes@knoda.org)
 
  *hk_datasource is the basic class which represents a resultquery or a table of a database.
@@ -68,7 +68,7 @@ class hk_datasource:public hk_data
         enum_tablemodes mode(void);
 /**
  *brings the datasource in insertmode. To add a new row call this function, set the data of the new row
- *as usual in the columns (i.e.with @ref hk_column::ashk_string ) and finally call
+ *as usual in the columns (i.e.with @ref hk_column::as_string ) and finally call
  *either @ref setmode_normal or @ref store_changed_data.
  */
         void    setmode_insertrow(void);
@@ -123,9 +123,9 @@ class hk_datasource:public hk_data
  *alters physically the table
  */
         bool alter_table_now(void);
-	
-	
-	
+
+
+
 	bool create_view_now(void);
 	bool alter_view_now(void);
 
@@ -224,7 +224,7 @@ class hk_datasource:public hk_data
 /**
  *if the datasource is enabled you get a list of all columns.
  *The fields can be reached by columns. See @ref hk_column for details.
-
+ * returns NULL if no columns are defined yet (e.g. non-enabled queries)
  */
         list<hk_column*>* columns(void);
 	list<hk_string>* columnnames(void);
@@ -257,7 +257,7 @@ class hk_datasource:public hk_data
  * set your own SQL statement, that will be executed when you enable the datasource.
  * if this datasource is of type "table" it will be automatically created.
  *@param  s is the sql statement
- *@param rawsql if true the sql statement will used as is (otherwise it may be changed to fit
+ *@param rawsql if true the sql statement will used as is (otherwise it may be changed )
  *@param registerchange if this class is part of a hk_presentation object (i.e. a form or a report) and registerchange
  *is true, then the changes will be stored when the hk_presentation object is closed.
  *sorting, filtering or depending on statements). So it is possible to execute driver specific
@@ -267,6 +267,10 @@ class hk_datasource:public hk_data
 virtual bool    set_query(hk_qbe*,bool registerchange=true);
         hk_string   sql(void) const;
         bool    is_rawsql(void);
+/**
+*returns the SQL statement as it is sent to the SQL server
+*/
+	hk_string   backendsql(void) const;
         typedef class
         {
             public:
@@ -435,6 +439,7 @@ See also @ref set_depending_on for details
  */
         void    set_automatic_data_update(bool u);
         bool    is_automatic_data_update(void);
+        
         void    set_sqldelimiter(const hk_string&);
 /**
  *sets the default date and timeformats for new created columns
@@ -592,18 +597,33 @@ list<hk_datasource*>* dependinglist(void);
   bool previous_enable_problems(void) const;
 
 list <referentialclass>* referenceslist(void);
-
+/** will try to add a new reference
+* to a table
+*/
 bool  add_reference(const referentialclass&);
 bool  drop_reference(const hk_string&);
 
 
-hk_string fieldorigin(const hk_string&);    
+hk_string fieldorigin(const hk_string&);
 hk_string totalfieldorigin(const hk_string&);
 void    reset_changed_data(void);
-bool has_changed(void) const;    
-
+bool has_changed(void) const;
+/**
+*returns a transformed column name in case the driver does only support e.g. ASCII characters
+*/
+hk_string systemcolumnname(const hk_string& n);
 
 static void set_enablefunction(voidfunction*, long counter);
+
+        bool   depending_on_datasource_deleterow_ok(void);
+        bool   depending_on_datasource_before_delete_row(void);
+        bool   depending_on_datasource_updaterow_ok(void);
+        bool check_store_changed_data();
+
+    
+    
+    
+    
     
     protected:
 /**
@@ -646,7 +666,7 @@ static void set_enablefunction(voidfunction*, long counter);
         virtual bool    driver_specific_alter_table_now(void){return false;}
         virtual bool    driver_specific_create_view_now(void);
         virtual bool    driver_specific_alter_view_now(void);
-        
+
 	virtual void    driver_specific_before_drop_table(void){}
         virtual void    filelist_changes(listtype t);
         virtual list<indexclass>* driver_specific_indices(void){return NULL;}
@@ -659,7 +679,7 @@ static void set_enablefunction(voidfunction*, long counter);
 	*/
         virtual void driver_specific_after_copy_table(){}
 	virtual bool driver_specific_load_view(){return false;}
-	virtual void driver_specific_create_viewsql(){}// used for ds_view 
+	virtual void driver_specific_create_viewsql(){}// used for ds_view
 	virtual bool driver_specific_add_reference(const referentialclass &);
 	virtual bool driver_specific_drop_reference(const hk_string& );
 	virtual void driver_specific_referenceslist(void);
@@ -669,9 +689,6 @@ static void set_enablefunction(voidfunction*, long counter);
         void    depending_on_datasource_data_has_changed(void);
         void    depending_on_datasource_after_store_changed_data(void);
         void    depending_on_datasource_insertmode(void);
-        bool   depending_on_datasource_deleterow_ok(void);
-        bool   depending_on_datasource_before_delete_row(void);
-        bool   depending_on_datasource_updaterow_ok(void);
         void   depending_on_datasource_before_update_row(void);
         void   depending_on_datasource_after_update_row(void);
         void    reload_data(bool take_changed_data=false);
@@ -706,7 +723,7 @@ static void set_enablefunction(voidfunction*, long counter);
                 bool primary;
                 bool notnull;
         } class_altercolumns;
-        
+
 	list<class_altercolumns>       p_altercolumns;
         list<hk_string>       p_deletecolumns;
         list<hk_column*>       p_newcolumns;
@@ -745,9 +762,9 @@ static void set_enablefunction(voidfunction*, long counter);
 		    list <fieldoriginclass> fieldpart;
 		    list <pair<hk_string,hk_string> > tablepart;
         } struct_parsed_sql;
-	
-	
-	
+
+
+
         struct_parsed_sql* p_parsed_sql;
 	void parse_fieldpart(void);
 	void parse_tablepart(void);
@@ -769,17 +786,7 @@ static void set_enablefunction(voidfunction*, long counter);
 	hk_string delimit_identifier(const hk_string&);
         static voidfunction* p_enablefunction;// a  function that will be called during enable
 	// used from the GUI, example: KDE should call processEvents to refresh display
-        static long p_enablefunctioncounter;     
-    private:
-
-        enum_tablemodes p_mode;
-        hk_string   p_original_sql;
-        unsigned long p_length;
-        bool    p_has_changed;
-        list<hk_dsvisible*> p_visibles;
-        list<hk_datasource*> p_dependinglist;
-        hk_datasource*  p_depending_on_datasource;
-        bool    p_depending_on_datasource_react_on_changed_data;
+        static long p_enablefunctioncounter;
         void    inform_before_row_change(void);
 	void    inform_visible_objects_batch_goto_next(void);
 	void    inform_visible_objects_batch_goto_previous(void);
@@ -797,6 +804,8 @@ static void set_enablefunction(voidfunction*, long counter);
         void    inform_depending_ds_store_data(void);
         void    inform_depending_ds_data_has_changed(void);
         void    inform_depending_ds_after_store_changed_data(void);
+        void    inform_depending_ds_before_update_row(void);
+        void    inform_depending_ds_after_update_row(void);
         void    inform_depending_ds_disable(void);
         void    inform_depending_ds_enable(void);
         void    inform_depending_ds_insertmode(void);
@@ -805,7 +814,6 @@ static void set_enablefunction(voidfunction*, long counter);
         void    inform_visible_objects_insertmode(void);
         void    inform_when_table_structure_changes(void);
         void    inform_when_indexlist_changes(void);
-        void    mark_visible_objects_as_not_handled(void);
 	void execute_visible_object_script_before_update(void);
 	void execute_visible_object_script_after_update(void);
 	void execute_visible_object_before_row_change(void);
@@ -814,6 +822,22 @@ static void set_enablefunction(voidfunction*, long counter);
 	void execute_visible_object_after_delete(void);
 	void execute_visible_object_before_insert(void);
 	void execute_visible_object_after_insert(void);
+
+
+        list<hk_datasource*> p_dependinglist;
+        enum_tablemodes p_mode;
+        list<referentialclass> p_references;
+	hk_string new_referencename(const referentialclass&);
+	bool is_newreferencename(const hk_string&);
+
+    private:
+        hk_string   p_original_sql;
+        unsigned long p_length;
+        bool    p_has_changed;
+        list<hk_dsvisible*> p_visibles;
+        hk_datasource*  p_depending_on_datasource;
+        bool    p_depending_on_datasource_react_on_changed_data;
+        void    mark_visible_objects_as_not_handled(void);
 	void set_columns_tableorigin(void);
 	hk_string columntablepart(const hk_string&);
         hk_presentation* p_presentation;
