@@ -1,5 +1,5 @@
 // ****************************************************************************
-// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>  
+// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>
 // This file is part of the hk_classes library.
 // This file may be distributed and/or modified under the terms of the
 // GNU Library Public License version 2 as published by the Free Software
@@ -8,7 +8,7 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 // ****************************************************************************
-//$Revision: 1.55 $
+//$Revision: 1.58 $
 #include "hk_dsgrid.h"
 #include "hk_form.h"
 #include "hk_column.h"
@@ -22,7 +22,6 @@ hk_dsgrid::hk_dsgrid(hk_form* form):hk_dsvisible(form)
     hkclassname("gridvisible");
     hkdebug("hk_dsgrid::constructor");
 #endif
-    p_dummycolumn=new hk_dsgridcolumn();
     p_automatic_columns=true;
     p_delete_nonexisting_columns=false;
     p_add_new_columns=false;
@@ -37,7 +36,6 @@ hk_dsgrid::~hk_dsgrid(void)
 #ifdef HK_DEBUG
     hkdebug("hk_dsgrid::destructor");
 #endif
-    delete p_dummycolumn;
     clear_cols();
 }
 
@@ -62,7 +60,7 @@ bool hk_dsgrid::columns_new_created(void)
         list<hk_column*>* dscols=datasource()->columns();
         if (dscols!=NULL)
         {
-            resize_cols(dscols->size()); 
+            resize_cols(dscols->size());
             vector<hk_dsgridcolumn*>::iterator cols_it=p_cols.begin();
             for (list<hk_column*>::iterator it=dscols->begin();it!=dscols->end();it++)
             {
@@ -270,7 +268,7 @@ hk_dsgridcolumn* hk_dsgrid::gridcolumn(unsigned int  c)
     }
     else
     {
-        return p_dummycolumn;
+        return NULL;
     }
 }
 
@@ -520,7 +518,7 @@ void hk_dsgrid::set_gridcolumns(list<hk_string>&sl)
         gridcolumn_exists::searchvalue=(*it);
         vector<hk_dsgridcolumn*>::iterator where=find_if(p_cols.begin(),p_cols.end(),gridcolumn_exists());
         if (where!=p_cols.end())
-        {   
+        {
             newcols[k]->set_displayname((*where)->displayname());
             newcols[k]->set_columntype((*where)->columntype());
             newcols[k]->set_columnwidth((*where)->columnwidth());
@@ -539,7 +537,7 @@ void hk_dsgrid::set_gridcolumns(list<hk_string>&sl)
 
     	    newcols[k]->set_on_getfocus_action((*where)->on_getfocus_action(),true,true);
 	    newcols[k]->set_on_loosefocus_action((*where)->on_loosefocus_action(),true,true);
-            
+
 	    if ((*where)->use_defaultvalue())
                 newcols[k]->set_defaultvalue((*where)->defaultvalue());
         }
@@ -595,6 +593,21 @@ list <hk_string>* hk_dsgrid::nonvisible_columns(void)
                 it++;
             }
         }
+        else
+        {
+           list <hk_string>* dscolnm=datasource()->columnnames();
+            list<hk_string>::iterator it=dscolnm->begin();
+            while (it!=dscolnm->end())
+            {
+                gridcolumn_exists::searchvalue=(*it);
+                vector<hk_dsgridcolumn*>::iterator where=find_if(p_cols.begin(),p_cols.end(),gridcolumn_exists());
+                if (where==p_cols.end())
+                {
+                    p_nonvisiblecols.insert(p_nonvisiblecols.end(),(*it));
+                }
+                it++;
+            }
+        }
     }
     return &p_nonvisiblecols;
 }
@@ -630,6 +643,7 @@ bool hk_dsgrid::load_table(void)
 #endif
     if (datasource()==NULL) return false;
     hk_string res=u2l(datasource()->database()->load(datasource()->name(),ft_table));
+    if (!p_presentation)datasource()->set_automatic_data_update(datasource()->database()->is_automatic_data_update());
     if (res.size()==0)return false;
 
     hk_string g;
