@@ -1,5 +1,5 @@
 // ****************************************************************************
-// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>  
+// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>
 // This file is part of the hk_classes library.
 // This file may be distributed and/or modified under the terms of the
 // GNU Library Public License version 2 as published by the Free Software
@@ -8,13 +8,16 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 // ****************************************************************************
-//$Revision: 1.63 $
+//$Revision: 1.71 $
 #include "hk_form.h"
 #include "hk_visible.h"
 #include "hk_dsdatavisible.h"
 #include "hk_dsgrid.h"
 #include "hk_database.h"
 #include "hk_dsvisible.h"
+#include "hk_dsdate.h"
+#include "hk_dsimage.h"
+#include "hk_tabvisible.h"
 #include "hk_dsdatavisible.h"
 #include "hk_dslineedit.h"
 #include "hk_dsrowselector.h"
@@ -30,11 +33,6 @@
 #define VERSION UNKNOWN
 #endif
 
-#ifdef HAVE_SSTREAM
-#include <sstream>
-#else
-#include <strstream.h>
-#endif
 
 
 class hk_formprivate
@@ -48,6 +46,7 @@ p_subform=NULL;
         list<hk_visible*>   p_visibles;
 	hk_form* p_masterform;
 	hk_subform* p_subform;
+	hk_string p_loadformname;
 
 };
 
@@ -110,11 +109,11 @@ void hk_form::add_visible(hk_visible* v)
     if (v==NULL) return;
     register_object(v);
     p_private->p_visibles.insert(p_private->p_visibles.end(),v);
-    
+
     if (v->type()==hk_visible::textlabel) return;
     p_viewdata->p_taborder.push_back(v->p_vupn);
-    
-    if (mode()==designmode) 
+
+    if (mode()==designmode)
         p_designdata->p_taborder.push_back(v->p_vupn);
 
 }
@@ -127,14 +126,14 @@ void hk_form::remove_visible(hk_visible* v)
 #endif
     if (!v) return;
     p_private->p_visibles.remove(v);
-    
-    
+
+
     if (mode()==designmode)
     {
       p_designdata->p_taborder.remove(v->p_vupn);
     }
     p_viewdata->p_taborder.remove(v->p_vupn);
-    
+
 }
 
 
@@ -311,6 +310,64 @@ hk_label*   hk_form::new_label(void)
 }
 
 
+hk_dsimage*   hk_form::new_image(void)
+{
+#ifdef HK_DEBUG
+    hkdebug("hk_form::new_image");
+#endif
+    if (mode()==viewmode)return NULL;
+    hk_dsimage* g = widget_specific_new_image();
+    if (g!=NULL)
+    {
+        add_visible(g);
+        g->set_presentationdatasource(presentationdatasource());
+        set_has_changed();
+        g->set_foregroundcolour(foregroundcolour());
+        //g->set_backgroundcolour(backgroundcolour());
+    }
+    return g;
+
+}
+
+
+hk_dsdate*   hk_form::new_date(void)
+{
+#ifdef HK_DEBUG
+    hkdebug("hk_form::new_date");
+#endif
+    if (mode()==viewmode)return NULL;
+    hk_dsdate* g = widget_specific_new_date();
+    if (g!=NULL)
+    {
+        add_visible(g);
+        set_has_changed();
+        g->set_foregroundcolour(foregroundcolour());
+        g->set_backgroundcolour(backgroundcolour());
+    }
+    return g;
+
+}
+
+
+hk_tabvisible*   hk_form::new_tabvisible(void)
+{
+#ifdef HK_DEBUG
+    hkdebug("hk_form::new_tabvisible");
+#endif
+    if (mode()==viewmode)return NULL;
+    hk_tabvisible* g = widget_specific_new_tabvisible();
+    if (g!=NULL)
+    {
+        add_visible(g);
+        set_has_changed();
+        g->set_foregroundcolour(foregroundcolour());
+        g->set_backgroundcolour(backgroundcolour());
+    }
+    return g;
+
+}
+
+
 void        hk_form::set_designsize(unsigned int w, unsigned int h,bool registerchange)
 {
 #ifdef HK_DEBUG
@@ -346,12 +403,13 @@ bool  hk_form::save_form(const hk_string& n,bool ask)
 
     }
     if (n.size()>0) set_name(n);
-
+    hk_database::enum_storage storage=hk_database::st_overwrite;
     reset_has_changed();
     if (name().size()==0) if (!ask_name())return false;
+    if (p_private->p_loadformname==name()) storage=hk_database::st_storechanges;
     if (database()->storagemode(ft_form)==hk_database::local)
     {
-      ofstream* p_save=database()->savestream(name(),ft_form,ask);
+      ofstream* p_save=database()->savestream(storage,name(),ft_form,ask,true,false);
       if (p_save==NULL) return false;
       savedata(*p_save);
       p_save->close();
@@ -359,18 +417,15 @@ bool  hk_form::save_form(const hk_string& n,bool ask)
     }
     else
     { // store central
-#ifdef HAVE_SSTREAM
-       stringstream* p_save;
-#else
-       strstream*  p_save;
-#endif
+       STRINGSTREAM*  p_save;
       p_save=database()->savestringstream(ft_form);
       if (p_save==NULL) return false;
       savedata(*p_save);
       hk_string s=p_save->str();
-      database()->save(s,name(),ft_form,ask);
+      database()->save(storage,s,name(),ft_form,ask);
       delete p_save;
     }
+    p_private->p_loadformname=name();
     reset_has_changed();
     return true;
 }
@@ -387,7 +442,7 @@ void hk_form::load_form(const hk_string& n)
         return ;
 
     }
-    
+
     if (has_changed())
     {
         save_form();
@@ -398,6 +453,7 @@ void hk_form::load_form(const hk_string& n)
     hk_dsmodevisible::set_mode(designmode);
     if (name().size()==0) if (!ask_name())return ;
     hk_string res =database()->load(name(),ft_form);
+    p_private->p_loadformname=n;
     if (res.size()==0) show_warningmessage(hk_translate("Bug: form is empty!"));
     if ( res.find("encoding=\"UTF-8\"")<res.size() ) res=u2l(res);
 
@@ -443,7 +499,7 @@ void    hk_form::savedata(ostream& s)
         set_tagvalue(s,"TABORDER",*tit);
       ++tit;
     }
-    
+
     end_mastertag(s,ftag);
 }
 
@@ -473,7 +529,7 @@ void    hk_form::loaddata(const hk_string& definition)
     }
 // end compatibility lines
     int i=1;
-    
+
     while (get_tagvalue(definition,"FORMOBJECT",value,i))
     {
         get_tagvalue(value,"VISIBLETYPE",buffer2);
@@ -516,6 +572,12 @@ hk_visible* hk_form::new_object(const hk_string& name)
                             if (name=="TEXTLABEL") return new_label();
                             else
                             if (name=="SUBFORM") return new_subform();
+                            else
+                            if (name=="IMAGE") return new_image();
+                            else
+                            if (name=="DATE") return new_date();
+                            else
+                            if (name=="TABVISIBLE") return new_tabvisible();
 
                                 return NULL;
 }
@@ -574,6 +636,10 @@ hk_visible* hk_form::get_visible(const hk_string &identifier)
 
 
 
+
+
+
+
 void hk_form::bulk_operation(enum_bulkoperation bulkoperation)
 {
     list<hk_visible*>::iterator it=p_private->p_visibles.begin();
@@ -602,16 +668,16 @@ void hk_form::bulk_operation(enum_bulkoperation bulkoperation)
 
 
 bool    hk_form::set_mode(enum_mode s)
-{
+{// cerr <<"hk_form::set_mode: "<<(s==viewmode?"viewmode":"designmode")<<endl;
     list<hk_visible*>::iterator it=p_private->p_visibles.begin();
 	while (it!=p_private->p_visibles.end())
         {
 	    if (s==viewmode&& (*it)->type()==subform)
-	    
+
 	    {
 	      hk_subform* s=static_cast<hk_subform*>(*it);
 	      s->internal_set_datasourcevalues();
-	    } 
+	    }
 	    ++it;
         }
     if (s==viewmode)
@@ -621,19 +687,30 @@ bool    hk_form::set_mode(enum_mode s)
     if (s==viewmode)
      {
 
-       if (!action_on_open())
+       if (!presentationmode_changed())
          {
-	    if (p_runtime_only)set_mode(designmode);
+         cerr <<"form action_on_open failed"<<endl;
+	    if (!p_runtime_only)set_mode(designmode);
 	   return false;
 	 }
-     } 
+     }
 
 	while (it!=p_private->p_visibles.end())
         {
-	    (*it)->presentationmode_changed();
+	    if (!(*it)->presentationmode_changed())
+	    {
+         cerr <<"visible action_on_open failed"<<endl;
+	    if (!p_runtime_only)set_mode(designmode);
+	    return false;
+	    }
 	    ++it;
         }
-    if (s==designmode) action_on_close();
+    if (s==designmode)
+    {
+     if (!p_while_errorhandling)
+       action_on_close();
+     p_while_errorhandling=false;
+    }
     return res;
 
 }
@@ -691,13 +768,13 @@ void hk_form::set_taborder(list<int> t,bool registerchange, bool force_setting)
        if (v && v->type()!=textlabel) l.push_back(*it);
        ++it;
      }
-     
+
      if (allow_datachanging(force_setting))
     {
      p_designdata->p_taborder=l;
     }
      p_viewdata->p_taborder=l;
-     
+
     has_changed(registerchange);
 
 }
@@ -716,11 +793,11 @@ hk_visible* hk_form::taborder_previous(hk_visible* v)
   if (!v) return NULL;
   list<int> t= taborder();
   list<int>::reverse_iterator it=find(t.rbegin(),t.rend(),v->p_vupn);
-  if (it==t.rend() && !p_private->p_visibles.empty()) 
+  if (it==t.rend() && !p_private->p_visibles.empty())
      return *(p_private->p_visibles.rbegin());
  ++it;
-  
-  if (it==t.rend() && !p_private->p_visibles.empty()) 
+
+  if (it==t.rend() && !p_private->p_visibles.empty())
   {
      cerr <<"taborder: am Ende angekommen "<<endl;
      if (is_subform())
@@ -729,25 +806,25 @@ hk_visible* hk_form::taborder_previous(hk_visible* v)
       }
       else
      it=t.rbegin();
-  }   
-       
-     
-  
+  }
+
+
+
   hk_visible* r=get_visible(*it);
-  
+
   if (r&&r->type()==subform)
     {
       hk_subform*s=dynamic_cast<hk_subform*>(r);
       if (s && s->subform())
         r=s->subform()->last_tabobject();
     }
-  
-  
-  if (!r && !p_private->p_visibles.empty()) 
+
+
+  if (!r && !p_private->p_visibles.empty())
      return *(p_private->p_visibles.rbegin());
   return r;
 }
-  
+
 
 
 hk_visible* hk_form::taborder_next(hk_visible* v)
@@ -755,24 +832,24 @@ hk_visible* hk_form::taborder_next(hk_visible* v)
   if (!v) return NULL;
   list<int> t= taborder();
   list<int>::iterator it=find(t.begin(),t.end(),v->p_vupn);
-  if (it==t.end() && !p_private->p_visibles.empty()) 
+  if (it==t.end() && !p_private->p_visibles.empty())
   {
-     
+
      return first_tabobject();
-  }   
+  }
  ++it;
-  
-  if (it==t.end() && !p_private->p_visibles.empty()) 
+
+  if (it==t.end() && !p_private->p_visibles.empty())
   {
-     
+
      if (is_subform())
-      { 
+      {
         return p_private->p_masterform->taborder_next(this->subformobject());
       }
       else
      it=t.begin();
-  }   
-  
+  }
+
   hk_visible* r=get_visible(*it);
   if (r&&r->type()==subform)
     {
@@ -798,10 +875,10 @@ hk_visible* hk_form::first_tabobject()
 
   if (!result)
   {
-  if ( !p_private->p_visibles.empty()) 
+  if ( !p_private->p_visibles.empty())
      result= *(p_private->p_visibles.begin());
-  }  
-  
+  }
+
   return result;
 }
 
@@ -817,10 +894,10 @@ hk_visible* hk_form::last_tabobject()
 
   if (!result)
   {
-  if (!p_private->p_visibles.empty()) 
+  if (!p_private->p_visibles.empty())
      result= *(p_private->p_visibles.begin());
-  }  
-  
+  }
+
   return result;
 }
 
@@ -850,3 +927,11 @@ hk_subform* hk_form::subformobject() const
 }
 
 
+int hk_form::screen_width()
+{
+return -1;
+}
+int hk_form::screen_height()
+{
+return -1;
+}
