@@ -1,5 +1,5 @@
 // ****************************************************************************
-// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>  
+// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>
 // This file is part of the hk_classes library.
 // This file may be distributed and/or modified under the terms of the
 // GNU Library Public License version 2 as published by the Free Software
@@ -8,11 +8,17 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 // ****************************************************************************
-//$Revision: 1.33 $
+//$Revision: 1.42 $
 #include "hk_importcsv.h"
 #include "hk_database.h"
 #include "hk_column.h"
 #include "hk_datasource.h"
+#include "hk_datetime.h"
+#include <iconv.h>
+#include <locale.h>
+#include <nl_types.h>
+#include <langinfo.h>
+
 hk_importcsv::hk_importcsv(void):hk_dsvisible()
 {
 #ifdef HK_DEBUG
@@ -27,8 +33,13 @@ hk_importcsv::hk_importcsv(void):hk_dsvisible()
     p_firstrow_contains_fieldnames=true;
     p_create_new_table=true;
     p_datamode=true;
-    p_notcancelimport=true;
+    p_cancelimport=true;
     p_overwrite=false;
+    p_progressdialog=NULL;
+    p_datetimeformat=defaultdatetimeformat();
+    p_dateformat=defaultdateformat();
+    p_timeformat=defaulttimeformat();
+    p_detect_autoinc=true;
 }
 
 
@@ -51,7 +62,7 @@ bool hk_importcsv::execute(enum_interaction enuminteractive)
     {
       show_warningmessage(hk_translate("Cannot import as you run in runtime only mode !"));
       return false;
-    
+
     }
     if (datasource()==NULL) return false;
     p_datamode=true;
@@ -74,10 +85,18 @@ bool hk_importcsv::execute(enum_interaction enuminteractive)
 #ifdef HK_DEBUG
     hkdebug("hk_importcsv::execute while");
 #endif
-    p_notcancelimport=true;
-    while (!p_filestream->eof()&&p_notcancelimport)
+    p_cancelimport=false;
+    std::streampos spos=p_filestream->tellg();
+    p_filestream->seekg(0,std::ios::end);
+    unsigned long maxsize=p_filestream->tellg();
+    unsigned long r=0;
+    p_filestream->seekg(spos);
+    while (!p_filestream->eof()&&!p_cancelimport)
     {
+     if (p_progressdialog)
+        p_cancelimport=p_progressdialog(r,maxsize,hk_translate("Importing data ..."));
         get_line(*p_filestream,sbuffer);
+        r=p_filestream->tellg();
         create_valuelist(sbuffer);
         unsigned int z=p_columnlist.size();
         if (z>datasource()->columns()->size())z=datasource()->columns()->size();
@@ -95,8 +114,17 @@ bool hk_importcsv::execute(enum_interaction enuminteractive)
             {
                 if ((*col_it).col!=NULL)
                 {
-                    if (!((*value_it)=="" &&is_numerictype((*col_it).col)))
-		    (*col_it).col->set_asstring((*value_it),true,false);
+                    if (!(is_numerictype((*col_it).col)))
+		    (*col_it).col->set_asstring((*value_it),true,true);
+		    else
+		    if ((*value_it)!="")
+		    {
+		     // handle numeric_types
+		     if (is_integertype((*col_it).col))
+		    (*col_it).col->set_asinteger((long int)standardstring2double((*value_it),p_locale));
+		    else
+		    (*col_it).col->set_asdouble(standardstring2double((*value_it),p_locale));
+		    }
                 }
                 col_it++;
                 value_it++;
@@ -105,11 +133,12 @@ bool hk_importcsv::execute(enum_interaction enuminteractive)
 
         i++;
         datasource()->store_changed_data();
-        p_notcancelimport=widget_specific_after_new_row();
+        p_cancelimport=!widget_specific_after_new_row();
     }
 #ifdef HK_DEBUG
     hkdebug("hk_importcsv::execute while ENDE");
 #endif
+
     reset();
 
     return true;
@@ -130,17 +159,20 @@ void hk_importcsv::get_line(ifstream&s ,hk_string& b)
       {
         inbrackets=!inbrackets;
       }
-      
+
      if (buf==p_rowdelimiter)
       {
         if (!inbrackets) noreturn=false;
-      } 
-     
+      }
+
      if (noreturn)   b=b+c;
      if (r==0) noreturn=false;
-      
-      
+
+
    }
+    hk_string use_locale=(p_charset.size()==0?nl_langinfo(CODESET):p_charset);
+   hk_string r=smallstringconversion(b,p_charset,"");
+   if (r.size()>0) b=r;
 }
 
 
@@ -177,7 +209,24 @@ bool hk_importcsv::initialize_table(enum_interaction e)
             create_automatic_columns();
         }
     }
-    
+
+    if (!p_create_new_table &&datasource()->database()->table_exists(datasource()->name()))
+    {
+	create_automatic_columns();
+    p_accessmode=datasource()->accessmode();
+    if (datasource()->is_enabled()) datasource()->disable();
+    datasource()->set_accessmode(hk_datasource::batchwrite);
+    if (!datasource()->is_enabled())datasource()->enable();
+    if (!datasource()->is_enabled())
+    {
+        reset();
+
+        show_warningmessage(hk_translate("Datasource could not be enabled"));
+        return false;
+    }
+      return true;
+    }
+
     if (p_create_new_table||(!p_create_new_table &&!datasource()->database()->table_exists(datasource()->name())))
     {
         if (datasource()->database()->table_exists(datasource()->name()))
@@ -198,7 +247,6 @@ bool hk_importcsv::initialize_table(enum_interaction e)
         get_line(*p_filestream,sbuffer);           //erste Datenzeile einlesen
         if (!p_firstrow_contains_fieldnames &&p_columnlist.size()==0)
         {
-
             create_valuelist(sbuffer);
             create_automatic_columns();
         }
@@ -210,7 +258,6 @@ bool hk_importcsv::initialize_table(enum_interaction e)
 #ifdef HK_DEBUG
         hkdebug("hk_importcsv::initialize_table vor while");
 #endif
-
         vector<colstruct>::iterator colit=p_columnlist.begin();
         vector<hk_string>::iterator it=p_valuelist.begin();
         while (it!=p_valuelist.end()&&colit!=p_columnlist.end())
@@ -224,20 +271,12 @@ bool hk_importcsv::initialize_table(enum_interaction e)
             hkdebug("angelegte Spalte: ",col->name());
 #endif
 
-            int c=interpret_columntype((*it)); //cerr <<"TYP:  ";
-            switch (c)
-            {
-                case 1:col->set_columntype(hk_column::floatingcolumn);//cerr<<"floating";
-                break;
-                case 2:col->set_columntype(hk_column::integercolumn);//cerr<<"integer";
-                break;
-                case 3:col->set_columntype(hk_column::boolcolumn);//cerr<<"bool";
-                break;
-                default:col->set_columntype(hk_column::textcolumn);//cerr<<"text";
-            }
-//cerr <<endl;
+            col->set_columntype(interpret_columntype((*it)));
             if ((*it).size()>255)col->set_columntype(hk_column::memocolumn);
             else col->set_size(255);
+            if (col->columntype()==hk_column::integercolumn)
+               (*colit).currentvalue=localestring2int((*it));
+            (*colit).col=col;
 #ifdef HK_DEBUG
             hkdebug("initialize_table ++");
 #endif
@@ -246,6 +285,88 @@ bool hk_importcsv::initialize_table(enum_interaction e)
 //         if (colit!=p_columnlist.end())
             colit++;
         }                                         //end while
+
+        const int maxpreview=50;
+        bool eof=false;
+        int i=0;
+#ifdef HK_DEBUG
+            hkdebug("Start type preview");
+#endif
+     for (i=0;i<maxpreview &&!eof;++i)
+     {
+        if (!p_filestream->eof())
+        {
+        get_line(*p_filestream,sbuffer);           //zweite bis maxpreview Datenzeile einlesen
+        p_datamode=false;
+        create_valuelist(sbuffer);
+        p_datamode=true;
+
+        it=p_valuelist.begin();
+        colit=p_columnlist.begin();
+        while (it!=p_valuelist.end()&&colit!=p_columnlist.end())
+        {
+#ifdef HK_DEBUG
+            hkdebug("in while begin");
+#endif
+            col=(*colit).col;
+            hk_column::enum_columntype ctype=interpret_columntype((*it));
+            //cerr <<"INTERPRETIERTER COLUMNTYPE akuteller Wert:"<<*it<<" "<<ctype<<endl;
+            if (col && ctype!=col->columntype())
+            {
+            if (ctype==hk_column::floatingcolumn && col->columntype()==hk_column::integercolumn
+            ||ctype==hk_column::integercolumn && col->columntype()==hk_column::floatingcolumn
+            )
+            col->set_columntype(hk_column::floatingcolumn);
+            else
+             if(col->columntype()!=hk_column::memocolumn)
+             {
+               col->set_columntype(hk_column::textcolumn);
+#ifdef HK_DEBUG
+		hkdebug("Fix data type to textcolumn, column:",(*colit).colname);
+#endif
+             }
+
+
+
+            if ((*it).size()>255)col->set_columntype(hk_column::memocolumn);
+            else col->set_size(255);
+            } //end different columntypes
+#ifdef HK_DEBUG
+            hkdebug("initialize_table ++");
+#endif
+            int v=localestring2int((*it));
+            //the following is needed to detect autoinc
+            if (col->columntype()==hk_column::integercolumn && v>(*colit).currentvalue &&(*colit).currentvalue>0)
+               (*colit).currentvalue=v;
+            else
+		(*colit).currentvalue=-1;
+	    //end detect autoinc part
+            it++;
+            colit++;
+        }                                         //end while
+        }
+        else eof=true;//eof
+
+    } // end for
+
+         colit=p_columnlist.begin();
+         bool has_autoinc=false;
+        if (p_detect_autoinc)
+        {
+        while (colit!=p_columnlist.end())
+        {
+	    if ((*colit).col->columntype()==hk_column::integercolumn&&(*colit).currentvalue>0&&!has_autoinc
+	    && i>5)
+	    {
+	       (*colit).col->set_columntype(hk_column::auto_inccolumn);
+	       has_autoinc=true;
+	    }
+	    (*colit).col=NULL;
+            colit++;
+        }                                         //end while
+        }
+
+
 #ifdef HK_DEBUG
         hkdebug("hk_importcsv::initialize_table NACH while");
 //datasource()->wanna_debug(true);
@@ -295,7 +416,7 @@ p_datamode=true;
 }
 
 
-void    hk_importcsv::set_filedefinition(const hk_string& fielddelimiter,const hk_string& betweenfields, const hk_string& rowdelimiter)
+void    hk_importcsv::set_filedefinition(const hk_string& fielddelimiter,const hk_string& betweenfields, const hk_string& rowdelimiter, const hk_string& datetimeformat, const hk_string& dateformat, const hk_string& timeformat,const hk_string& charset,const hk_string& locale)
 {
 #ifdef HK_DEBUG
     hkdebug("hk_importcsv::set_filedefinition");
@@ -303,6 +424,11 @@ void    hk_importcsv::set_filedefinition(const hk_string& fielddelimiter,const h
     p_textdelimiter=fielddelimiter;
     if (betweenfields.size()>0) p_betweenfields=betweenfields;
     p_rowdelimiter=rowdelimiter;
+    if (datetimeformat.size()>0)p_datetimeformat=datetimeformat;
+    if (timeformat.size()>0) p_timeformat=timeformat;
+    if (dateformat.size()>0) p_dateformat=dateformat;
+    p_charset=charset;
+    p_locale=locale;
 }
 
 
@@ -343,7 +469,7 @@ void hk_importcsv::set_append_rows(bool append_rows)
 
 
 
-void hk_importcsv::create_valuelist( hk_string& row)
+vector<hk_string>* hk_importcsv::create_valuelist( hk_string& row)
 {
 #ifdef HK_DEBUG
 //wanna_debug(true);
@@ -360,7 +486,7 @@ void hk_importcsv::create_valuelist( hk_string& row)
 #ifdef HK_DEBUG
         hkdebug("hk_importcsv::create_valuelist return am Beginn der whileschleife");
 #endif
-        return;
+        return &p_valuelist;
     }
     hk_string::size_type offset=0;
 //  hk_string::size_type beginposition=0;
@@ -368,7 +494,7 @@ void hk_importcsv::create_valuelist( hk_string& row)
     hk_string field;
 #ifdef HK_DEBUG
     hkdebug("Zeile: ",row);
-    hkdebug("Gesamtlnge: ",(int)row.size());
+    hkdebug("Gesamtlänge: ",(int)row.size());
     hkdebug("Z: ",row);
     hkdebug("Z: 12345678901234567890123456789012345678901234567890123456789012345678901234567890");
     hkdebug("Z: 0        1         2         3         4         5         6         7         8");
@@ -444,7 +570,7 @@ void hk_importcsv::create_valuelist( hk_string& row)
                 if (x == p_betweenfields ||x==p_rowdelimiter)
                 {
                                                  // cout <<"S_END_OF_QUOTED_FIELD  nach S_START"<<field<<" x="<<x<<endl;
-                    p_valuelist.insert(p_valuelist.end(),field);cerr <<"FIELD: "<<field<<endl;
+                    p_valuelist.insert(p_valuelist.end(),field);//cerr <<"FIELD: "<<field<<endl;
                     field = "";
                     state = S_START;
                 }
@@ -470,16 +596,18 @@ void hk_importcsv::create_valuelist( hk_string& row)
                 }
         }
     }
-
-    
-/*  vector<hk_string>::iterator it=p_valuelist.begin();
+#ifdef HK_DEBUG
+  if (wanna_debug())
+  {
+  vector<hk_string>::iterator it=p_valuelist.begin();
   while (it!=p_valuelist.end())
   {
-    cerr <<"Feld:'"<<(*it)<<endl<<endl;
+    cerr <<"Feld:'"<<(*it)<<"'"<<endl;
   ++it;
-  }  
-*/    
-    
+  }
+  }
+#endif
+return &p_valuelist;
 }
 
 
@@ -491,14 +619,13 @@ void    hk_importcsv::add_columnname(const hk_string& col)
     colstruct p_col;
 
 // remove leading and trailing spaces
-    hk_string::size_type b=col.find_first_not_of(" ");
-    hk_string::size_type e=col.find_last_not_of(" ");
-    if (e==hk_string::npos) e=col.size();
-    if (b!=hk_string::npos) p_col.colname=col.substr(b,e-b+1);
+
+    p_col.colname=trim(col);
 
 // replace interior spaces with underscores
     p_col.colname=replace_all(" ",p_col.colname,"_");
     p_col.col=NULL;
+    p_col.currentvalue=-1;
     p_columnlist.insert(p_columnlist.end(),p_col);
 }
 
@@ -517,14 +644,19 @@ void hk_importcsv::set_columns(void)
 #ifdef HK_DEBUG
     hkdebug("hk_importcsv::set_columns");
 #endif
+//cerr <<"set_columns:" <<datasource()<<endl;
     if (datasource()==NULL) return;
     vector<colstruct>::iterator it=p_columnlist.begin();
+    hk_string result; cerr <<"p_columnlist:"<<p_columnlist.size()<<endl;
     while (it!=p_columnlist.end())
     {
         (*it).col=datasource()->column_by_name((*it).colname);
 //cout <<"Suche nach Column: " << (*it).colname<<" "<< (!(*it).col?"nicht ":"")<<"erfolgreich"<<endl;
+	if (!(*it).col) result+=(*it).colname+"\n";
         it++;
     }
+   if (result.size()>0)
+    show_warningmessage(hk_translate("The following columns could not be found in the table:\n")+result);
 
 }
 
@@ -534,6 +666,23 @@ void    hk_importcsv::create_automatic_columns(void)
 #ifdef HK_DEBUG
     hkdebug("hk_importcsv::create_automatic_columns");
 #endif
+
+    if (!p_create_new_table &&datasource()->database()->table_exists(datasource()->name()))
+    {
+       list<hk_column*>* col=datasource()->columns();
+       list<hk_column*>::iterator it=col->begin();
+       while(it!=col->end())
+    {
+        add_columnname((*it)->name());
+
+        ++it;
+    }
+
+      return ;
+    }
+
+
+
     hk_string newfieldname;
     clear_columnlist();
     int zaehler=0;
@@ -569,8 +718,7 @@ bool hk_importcsv::before_columns_deleted(void)
     return true;
 }
 
-
-int hk_importcsv::interpret_columntype(hk_string& field)
+hk_column::enum_columntype hk_importcsv::interpret_columntype(hk_string& field)
 {
 #ifdef HK_DEBUG
     hkdebug("hk_importcsv::interpret_columntype");
@@ -578,22 +726,76 @@ int hk_importcsv::interpret_columntype(hk_string& field)
     int num=0;
     int komma=0;
     int alpha=0;
-   if (field=="TRUE"||field=="FALSE") return 3;
+    hk_datetime dt;
+    dt.set_datetimeformat(p_datetimeformat);
+    dt.set_dateformat(p_dateformat);
+    dt.set_timeformat(p_timeformat);
+   if (field=="TRUE"||field=="FALSE")
+   {
+#ifdef HK_DEBUG
+    hkdebug("interpreted datatype: boolcolumn");
+#endif
+    return hk_column::boolcolumn;
+   }
+   if (dt.set_datetime_asstring(field))
+   {
+#ifdef HK_DEBUG
+    hkdebug("interpreted datatype: datetimecolumn");
+#endif
+    return hk_column::datetimecolumn;
+   }
+   if (dt.set_date_asstring(field))
+   {
+#ifdef HK_DEBUG
+    hkdebug("interpreted datatype: datecolumn");
+#endif
+    return hk_column::datecolumn;
+   }
+   if (dt.set_time_asstring(field))
+   {
+#ifdef HK_DEBUG
+    hkdebug("interpreted datatype: timecolumn");
+#endif
+   return hk_column::timecolumn;
+   }
+
     for (unsigned int i=0;i<field.size();i++)
     {
 
         if (isdigit(field[i])) num++;
-        else if (field[i]=='.')komma++;
+        else if (field[i]=='.'||field[i]==',')komma++;
         else alpha++;
 
     }
 //  cout <<"interpretiere feldtyp: "<<field<<endl;
 //  cout <<"alpha: "<<alpha<<" komma: "<<komma<<" num: "<<num<<endl;
 
-    if (alpha>0||komma>1)return 0;
-    if (num>0) if (komma==1) return 1;
-    else return 2;
-    return 0;
+    if (alpha>0||komma>1)
+    {
+#ifdef HK_DEBUG
+    hkdebug("interpreted datatype: textcolumn, da komma >1");
+#endif
+    return hk_column::textcolumn;
+    }
+    if (num>0) if (komma==1)
+    {
+#ifdef HK_DEBUG
+    hkdebug("interpreted datatype: floatingcolumn");
+#endif
+
+     return hk_column::floatingcolumn;
+    }
+    else
+    {
+#ifdef HK_DEBUG
+    hkdebug("interpreted datatype: integercolumn");
+#endif
+    return hk_column::integercolumn;
+    }
+#ifdef HK_DEBUG
+    hkdebug("interpreted datatype: textcolumn default");
+#endif
+    return hk_column::textcolumn;
 
 }
 
@@ -621,8 +823,35 @@ void hk_importcsv::set_overwrite_table(bool o)
 
 }
 
+void    hk_importcsv::set_create_new_table(bool n)
+{
+   p_overwrite=!n;
+}
 
 bool hk_importcsv::overwrite_table(void)
 {
     return p_overwrite;
 }
+
+
+void hk_importcsv::set_detect_autoinc(bool v)
+{
+ p_detect_autoinc=v;
+}
+
+bool hk_importcsv::detect_autoinc(void) const
+{
+  return p_detect_autoinc;
+}
+
+void hk_importcsv::set_progressdialog(progress_dialogtype* progressdialog)
+{
+p_progressdialog=progressdialog;
+}
+
+progress_dialogtype* hk_importcsv::progressdialog(void)const
+{
+  return p_progressdialog;
+}
+
+
