@@ -1,5 +1,5 @@
 // ****************************************************************************
-// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>  
+// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>
 // This file is part of the hk_classes library.
 // This file may be distributed and/or modified under the terms of the
 // GNU Library Public License version 2 as published by the Free Software
@@ -8,7 +8,7 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 // ****************************************************************************
-//$Revision: 1.73 $
+//$Revision: 1.76 $
 
 #include "hk_presentation.h"
 #include "hk_database.h"
@@ -72,6 +72,7 @@ hk_presentation::hk_presentation(void):hk_dsmodevisible()
     p_private->p_sizetype=p_defaultsizetype;
     set_designsize(100,100);
     p_private->p_interpretername="python";
+    p_while_errorhandling=false;
 }
 
 
@@ -86,7 +87,7 @@ hk_presentation::~hk_presentation(void)
         p_datasource->p_presentation=NULL;
     }
     list<hk_datasource*>::iterator it=p_private->p_datasources.begin();
-    
+
     while (it!=p_private->p_datasources.end())
     {
         hk_datasource* d=(*it);
@@ -116,7 +117,7 @@ hk_presentation::~hk_presentation(void)
     }
 
     p_private->p_datasources.clear();
-    
+
     p_datasource=NULL;
     if (p_private->p_database!=NULL) p_private->p_database->presentation_remove(this);
    delete p_private;
@@ -149,7 +150,7 @@ long hk_presentation::add_datasource(hk_datasource* d)
 #ifdef HK_DEBUG
     hkdebug("hk_presentation::add_datasource");
 #endif
-    if (d==NULL) 
+    if (d==NULL)
      {
        show_warningmessage(hk_translate("Bug: tried to add empty datasource!"));
        return -1;
@@ -201,7 +202,7 @@ hk_datasource*  hk_presentation::get_datasource(const hk_string& description)
     list<hk_datasource*>::iterator it=p_private->p_datasources.begin();
     while (it!=p_private->p_datasources.end())
     {
-        if ( unique_datasourcename((*it)->p_dscounter)==description) 
+        if ( unique_datasourcename((*it)->p_dscounter)==description)
 	  return (*it);
         it++;
     }
@@ -216,7 +217,7 @@ hk_datasource*  hk_presentation::get_datasource_by_shortname(const hk_string& de
     list<hk_datasource*>::iterator it=p_private->p_datasources.begin();
     while (it!=p_private->p_datasources.end())
     {
-        if ( unique_shortdatasourcename((*it)->p_dscounter)==description) 
+        if ( unique_shortdatasourcename((*it)->p_dscounter)==description)
 	  return (*it);
         it++;
     }
@@ -298,6 +299,7 @@ void    hk_presentation::savedata(ostream& s)
     end_mastertag(s,"DATASOURCES");
 
     end_mastertag(s,tag);
+p_is_new=false;
 }
 
 
@@ -331,7 +333,7 @@ void    hk_presentation::loaddata(const hk_string& definition)
         if (buffer2=="QUERY")  dt=dt_query;
 	else
         if (buffer2=="VIEW")  dt=dt_view;
-	
+
         long int dint=new_datasource(dsname,dt);
         ds=get_datasource(dint);
         if (ds!=NULL)
@@ -345,7 +347,7 @@ void    hk_presentation::loaddata(const hk_string& definition)
     //if (p_private->p_datasources.size()==0) show_warningmessage(hk_translate("No datasources defined!")) ;
     get_tagvalue(definition,"HK_DSVISIBLE",value);
     hk_dsmodevisible::loaddata(value);
-
+p_is_new=false;
 }
 
 
@@ -413,7 +415,7 @@ void hk_presentation::enable_datasources(bool enable,hk_datasource::enum_accessm
             {
                 if ((*it)->is_enabled()) (*it)->disable();
                 (*it)->set_accessmode(mode);
-                //if (!((*it)->previous_enable_problems()))  
+                //if (!((*it)->previous_enable_problems()))
 		   (*it)->enable();
             }
         }
@@ -447,7 +449,7 @@ void hk_presentation::database_has_vanished(void)
 
 void hk_presentation::before_source_vanishes(void)
 {
-    
+
     //hk_dsmodevisible::before_source_vanishes();
     //p_datasources.remove(datasource());
 }
@@ -648,10 +650,10 @@ void hk_presentation::sizetype_changed(void)
 
 hk_presentation::enum_sizetype hk_presentation::defaultsizetype(void)
  {
- 
+
  return p_defaultsizetype;
  }
- 
+
 hk_interpreter* hk_presentation::interpreter()
 {
    if (p_private->p_interpreter)
@@ -679,6 +681,7 @@ return p_private->p_interpretername;
 
 void            hk_presentation::set_name(const hk_string& n)
 {
+ if (n!=p_private->p_name)p_is_new=true;
  p_private->p_name=n;
 }
 hk_string       hk_presentation::name(void) const
