@@ -1,5 +1,5 @@
 // ****************************************************************************
-// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>  
+// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>
 // This file is part of the hk_classes library.
 // This file may be distributed and/or modified under the terms of the
 // GNU Library Public License version 2 as published by the Free Software
@@ -9,6 +9,9 @@
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 // ****************************************************************************
 #include "hk_qbe.h"
+#include "hk_datasource.h"
+#include "hk_database.h"
+#include "hk_connection.h"
 
 class hk_qbeprivate
 {
@@ -29,7 +32,7 @@ hk_qbe::hk_qbe(void):hk_presentation()
 #ifdef HK_DEBUG
     hkdebug("hk_qbe::hk_qbe");
 #endif
-    p_private=new hk_qbeprivate;  
+    p_private=new hk_qbeprivate;
     p_presentationtype=hk_presentation::qbe;
 }
 
@@ -84,10 +87,10 @@ bool hk_qbe::create_sql(hk_string& sql)
      case qt_groupselect:sql=create_groupselect_sql(result);break;
      case qt_update:sql=create_update_sql(result);break;
      case qt_delete:sql=create_delete_sql(result);break;
-   
+
    }
    if (!result) cerr <<"hk_qbe: Warning error while creating sql statement"<<endl;
-   
+
    return result;
 
 }
@@ -123,7 +126,7 @@ void hk_qbe::savedata(ostream& s)
         case descending : buffer="DESCENDING";break;
       }
       set_tagvalue(s,"ORDER",buffer);
-      
+
       switch ((*it).functiontype)
       {
          case ft_condition : buffer="CONDITION";break;
@@ -135,7 +138,7 @@ void hk_qbe::savedata(ostream& s)
         case ft_avg : buffer="AVERAGE";break;
       }
       set_tagvalue(s,"FUNCTIONTYPE",buffer);
-      
+
       set_tagvalue(s,"SHOW",(*it).show);
       set_tagvalue(s,"UPDATEVALUE",(*it).updatevalue);
          start_mastertag(s,"CONDITIONS");
@@ -146,7 +149,7 @@ void hk_qbe::savedata(ostream& s)
 		++sit;
 	     }
          end_mastertag(s,"CONDITIONS");
- 
+
     end_mastertag(s,"QBEDEFINITION");
     ++it;
    }
@@ -189,12 +192,12 @@ void hk_qbe::loaddata(const hk_string& definition)
       get_tagvalue(value,"ALIAS",alias);
       if(get_tagvalue(value,"ORDER",buffer))
           {
-	     
+
 	     if (buffer=="ASCENDING") order=ascending;
-	     else 
+	     else
 	     if (buffer=="DESCENDING") order=descending;
           }
-      
+
       if (get_tagvalue(value,"FUNCTIONTYPE",buffer))
       {
           if (buffer=="GROUP") functiontype=ft_group;
@@ -208,17 +211,17 @@ void hk_qbe::loaddata(const hk_string& definition)
           if (buffer=="MAX") functiontype=ft_max;
 	  else
           if (buffer=="AVERAGE") functiontype=ft_avg;
-      }      
+      }
       get_tagvalue(value,"UPDATEVALUE",updatevalue);
       get_tagvalue(value,"SHOW",show);
-      
+
       int c=1;
       hk_string cvalue;
 	while (get_tagvalue(value,"CONDITION",buffer,c))
 	{
 	  conditionlist.insert(conditionlist.end(),buffer);
 	  ++c;
-	}  
+	}
 
 	++i;
      add_definition(field,table,alias,functiontype,order,show,updatevalue,conditionlist,false);
@@ -226,7 +229,7 @@ void hk_qbe::loaddata(const hk_string& definition)
    set_querytype(typebuffer,false);
 
    reset_has_changed();
-   
+
 }
 
 
@@ -262,7 +265,7 @@ void hk_qbe::add_definition(  hk_qbedataclass* cl,bool registerchange)
 
 
 void hk_qbe::add_definition(	const hk_string& field,
-			int table, 
+			int table,
 			const hk_string& alias,
 			enum_functiontype functiontype,
 			enum_order order,
@@ -275,9 +278,9 @@ void hk_qbe::add_definition(	const hk_string& field,
 #ifdef HK_DEBUG
     hkdebug("hk_qbe::add_definition(const hk_string& ...)");
 #endif
-  
+
   if (field.size()==0) return;
-  
+
   hk_qbedataclass c;
   c.field=field;
   c.table=table;
@@ -313,19 +316,19 @@ hk_string hk_qbe::create_where(void)
   for(unsigned int i=0;i<max;++i)
   {
    it=p_private->p_definitionlist.begin();
-   hk_string where; 
+   hk_string where;
    while (it!=p_private->p_definitionlist.end())
    {
     vector<hk_string> condi=(*it).conditions;
-    
+
     if (i<condi.size() && condi[i].size()>0)
     {
           if (where.size()>0) where+=") AND (";
 	  hk_qbedataclass* cl=&(*it);
-	  
+
     if (cl->table>-1)
      {
-	 
+
 	 switch (p_private->p_querytype)
   	  {
   		case qt_update:
@@ -334,19 +337,19 @@ hk_string hk_qbe::create_where(void)
 		  /*hk_datasource* ds=get_datasource(cl->table);
 		  if (ds)
 		  where+="\""+ds->name()
-	         +"\".\""+cl->field+"\" "+condi[i]; 
+	         +"\".\""+cl->field+"\" "+condi[i];
 		 */
-		 where+="\""+cl->field+"\" "+condi[i]; 
+		 where+="\""+cl->field+"\" "+condi[i];
 		  break;
 		  }
   		default:   where+="\""+unique_shortdatasourcename((cl->table))
-	         +"\".\""+cl->field+"\" "+condi[i]; 
+	         +"\".\""+cl->field+"\" "+condi[i];
   	  }
     }
-   }   
+   }
     ++it;
    }
-  if (where.size()>0) 
+  if (where.size()>0)
      {
      where="(("+where+"))";
      if (result.size()>0) result+=" OR "+where;
@@ -364,7 +367,7 @@ hk_string hk_qbe::create_where(void)
 }
 
 
-// 
+//
 
 
 
@@ -385,7 +388,7 @@ hk_string hk_qbe::create_depending_where(void)
         list<hk_string>::iterator this_ds;
         list<hk_string>::iterator master_ds;
         master_ds=(*it)->depending_on_masterfields()->begin();
-       
+
         for (this_ds=(*it)->depending_on_thisfields()->begin();this_ds!=(*it)->depending_on_thisfields()->end();this_ds++)
         {
             if (newwhere=="")
@@ -395,18 +398,18 @@ hk_string hk_qbe::create_depending_where(void)
             newwhere=newwhere+"\""+unique_shortdatasourcename((*it)->presentationnumber())+"\".\""+*this_ds+"\"=";
 	    newwhere=newwhere+"\""+unique_shortdatasourcename(p_depending_on_datasource->presentationnumber())+"\""
             +"."+"\""+*master_ds+"\"";
-	    
+
             master_ds++;
 
         }
 
     }                                             //endif depending_on_datasource !=NULL
-   
+
     ++it;
   }
   if (newwhere.size()>0) newwhere+=")";
-  
-  
+
+
   return newwhere;
 }
 
@@ -417,11 +420,12 @@ hk_string hk_qbe::create_from(void)
 #endif
 
 hk_string result;
-  
+
   list<hk_datasource*>::iterator it=datasources()->begin();
-  
+  bool needs_table_as=false;
   while (it!=datasources()->end())
   {
+  if (it==datasources()->begin()) needs_table_as=(*it)->database()->connection()->server_needs(hk_connection::NEEDS_TABLE_ALIAS_AS);
   if (result.size()>0) result+=" , ";
   switch (p_private->p_querytype)
   {
@@ -429,7 +433,9 @@ hk_string result;
   case qt_delete:
   		  result+="\""+(*it)->name()+"\"";
 		  break;
-  default: result+="\""+(*it)->name()+"\""+" AS "+"\""+unique_shortdatasourcename((*it)->presentationnumber())+"\"";
+  default: result+="\""+(*it)->name()+"\""
+           +(needs_table_as?" AS ":" ");
+           result+="\""+unique_shortdatasourcename((*it)->presentationnumber())+"\"";
   }
   ++it;
   }
@@ -462,17 +468,17 @@ hk_string hk_qbe::create_what(void)
   list<hk_qbe::hk_qbedataclass>::iterator it=p_private->p_definitionlist.begin();
   while (it!=p_private->p_definitionlist.end())
   {
-   
+
    if ((p_private->p_querytype==qt_select && (*it).show)||
        (p_private->p_querytype==qt_groupselect &&
       (*it).functiontype!=ft_condition ))
    {
      if (result.size()>0) result+=" , ";
      result+=fieldname(&(*it),true);
-   }  
+   }
   ++it;
   }
-  
+
   return result;
 
 }
@@ -489,15 +495,15 @@ hk_string hk_qbe::create_update_set(void)
   list<hk_qbe::hk_qbedataclass>::iterator it=p_private->p_definitionlist.begin();
   while (it!=p_private->p_definitionlist.end())
   {
-   
+
    if ((*it).updatevalue.size()>0 )
    {
      if (result.size()>0) result+=" , ";
      result+=fieldname(&(*it),true)+" = "+(*it).updatevalue;
-   }  
+   }
   ++it;
   }
-  
+
   return result;
 }
 
@@ -516,10 +522,10 @@ hk_string hk_qbe::create_order_by(void)
      if (result.size()>0) result+=" , ";
      result+=(*it).alias.size()==0?fieldname(&(*it)):"\""+(*it).alias+"\"";
      if ((*it).order==descending) result+=" DESC";
-   }  
+   }
   ++it;
   }
-  
+
   return result;
 
 }
@@ -538,10 +544,10 @@ hk_string hk_qbe::create_group_by(void)
    {
      if (result.size()>0) result+=" , ";
      result+=fieldname(&(*it));
-   }  
+   }
   ++it;
   }
-  
+
   return result;
 
 }
@@ -552,14 +558,14 @@ hk_string hk_qbe::create_group_by(void)
 
 void hk_qbe::widget_specific_querytype_has_changed(enum_querytype ,enum_querytype )
 {
-   
+
 }
 
 
 
 
 
-const list<hk_qbe::hk_qbedataclass>* hk_qbe::definitionlist(void) 
+const list<hk_qbe::hk_qbedataclass>* hk_qbe::definitionlist(void)
 {
    return &p_private->p_definitionlist;
 }
@@ -587,7 +593,7 @@ hk_string hk_qbe::create_select_sql(bool& valid)
        valid=false;
        return "";
      }
-     
+
   result=(p_private->p_distinct?"SELECT DISTINCT ":"SELECT ")+result+" FROM " +from;
   hk_string where=create_where();
   if (where.size()>0)
@@ -608,7 +614,7 @@ hk_string hk_qbe::create_select_sql(bool& valid)
 
 hk_string hk_qbe::create_groupselect_sql(bool& valid)
 {
-  
+
 #ifdef HK_DEBUG
     hkdebug("hk_qbe::create_groupselect_sql");
 #endif
@@ -626,9 +632,9 @@ hk_string hk_qbe::create_groupselect_sql(bool& valid)
        valid=false;
        return "";
      }
- 
-  
-  
+
+
+
   result=(p_private->p_distinct?"SELECT DISTINCT ":"SELECT ")+result+" FROM " +from;
   hk_string group_by=create_group_by();
   if (group_by.size()==0)
@@ -638,7 +644,7 @@ hk_string hk_qbe::create_groupselect_sql(bool& valid)
        return "";
      }
   result+=" GROUP BY "+group_by;
-  
+
   hk_string where=create_where();
   if (where.size()>0)
      {
@@ -650,7 +656,7 @@ hk_string hk_qbe::create_groupselect_sql(bool& valid)
        result+=" ORDER BY "+orderby;
      }
      valid=true;
-     
+
   return result;
 }
 
@@ -671,7 +677,7 @@ hk_string hk_qbe::create_update_sql(bool& valid)
        valid=false;
        return "";
      }
-     
+
   hk_string set=create_update_set();
   if (set.size()==0)
      {
@@ -679,15 +685,15 @@ hk_string hk_qbe::create_update_sql(bool& valid)
        valid=false;
        return "";
      }
-     
+
   hk_string result="UPDATE "+from+" SET "+set;
   hk_string where=create_where();
   if (where.size()>0)
      {
        result+=" WHERE "+where;
      }else cerr <<"WHERE.size()==0!"<<endl;
- 
-  
+
+
   valid=true;
   return result;
 }
@@ -715,9 +721,9 @@ hk_string hk_qbe::create_delete_sql(bool& valid)
      {
        result+=" WHERE "+where;
      }
- 
-    
-  
+
+
+
   valid=true;
   return result;
 }
@@ -741,13 +747,13 @@ hk_string hk_qbe::fieldname(hk_qbedataclass* cl,bool withalias)
 		  break;
 		  }
   default:   result="\""+unique_shortdatasourcename((cl->table))+"\"";
-     
+
   }
   if (result.size()>0)
   result+=cl->field=="*"?".*":".\""+cl->field+"\"";
   else
   result=cl->field=="*"?"*":"\""+cl->field+"\"";
-    
+
     if (p_private->p_querytype==qt_groupselect)
     {
      hk_string function;
@@ -767,7 +773,7 @@ hk_string hk_qbe::fieldname(hk_qbedataclass* cl,bool withalias)
   { // no table name set
     result=cl->field;
   }
-  
+
   if (cl->alias.size()>0&& withalias)
    {
      result+=" AS \""+cl->alias+"\"";
