@@ -1,5 +1,5 @@
 // ****************************************************************************
-// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>  
+// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>
 // This file is part of the hk_classes library.
 // This file may be distributed and/or modified under the terms of the
 // GNU Library Public License version 2 as published by the Free Software
@@ -8,7 +8,7 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 // ****************************************************************************
-// $Revision: 1.102 $
+// $Revision: 1.106 $
 #include "hk_report.h"
 #include "hk_reportsection.h"
 #include "hk_reportsectionpair.h"
@@ -27,19 +27,16 @@
 #else
 #define VERSION UNKNOWN
 #endif
-#ifdef HAVE_SSTREAM
-#include <sstream>
-#else
-#include <strstream.h>
-#endif
 
 class hk_reportprivate
 {
 public:
 hk_reportprivate()
-  { 
+  {
     p_while_executing=false;
     p_rowposition=0;
+    p_progressdialog=NULL;
+    p_locale="";
   }
 ~hk_reportprivate(){}
 hk_string p_on_new_page_action;
@@ -101,7 +98,11 @@ hk_string p_on_new_page_action;
         hk_reportsection* p_pagefooter;
         hk_reportsection* p_reportheader;
         hk_reportsection* p_reportfooter;
+        hk_string p_locale;
+        hk_string p_charset;
         bool p_while_executing;
+        progress_dialogtype* p_progressdialog;
+	hk_string p_loadreportname;
 };
 
 
@@ -438,7 +439,7 @@ bool hk_report::internal_execute(void)
             }
             hk_string st=(*sit)->columnname();
 	    if (datasource()->type()==hk_datasource::ds_query)
-	    { 
+	    {
 	       hk_datasource* d=datasource()->database()->new_resultquery();
 	       d->set_sql(datasource()->sql());
 	       d->set_filter("0=1");
@@ -456,8 +457,8 @@ bool hk_report::internal_execute(void)
 	       }
 	       else
 	       st=datasource()->fieldorigin(st);
-	    }   
-	    else st="\""+st+"\"";   
+	    }
+	    else st="\""+st+"\"";
             if (st.size()>0 &&!(*sit)->ascending_order()) st+=" DESC";
             if (sort.find(st)>sort.size())
             {
@@ -507,8 +508,8 @@ bool hk_report::internal_execute(void)
     {
         if (p_private->p_output==&cout)*outputstream()<<replace(p_private->p_filebegin);
 	if (!action_on_new_page())stop_execution();
-        if (p_private->p_pageheader!=NULL) 
-	if (!p_private->p_pageheader->actual_string()) 
+        if (p_private->p_pageheader!=NULL)
+	if (!p_private->p_pageheader->actual_string())
 	  stop_execution();
 	;
     }
@@ -528,6 +529,9 @@ bool hk_report::internal_execute(void)
     do
     {
 //print header sections
+     if (p_private->p_progressdialog)
+        p_private->p_cancelreport=p_private->p_progressdialog(p_private->p_rowposition,datasource()->max_rows(),hk_translate("Creating output ..."));
+
         vector<hk_reportsectionpair*>::iterator it=p_sectionpairs.begin();
         int p=0;
         bool printfootersection=true;
@@ -548,9 +552,9 @@ bool hk_report::internal_execute(void)
                         if (datasource())
 			 {
 			  datasource()->goto_previous();
-			 } 
+			 }
+
 
-			
                         while (c>=p+1)
                         {
                             hk_reportsection* f=p_sectionpairs[c-1]->footersection();
@@ -571,7 +575,7 @@ bool hk_report::internal_execute(void)
 				     {
 				       datasource()->goto_previous();
 
-				     }  
+				     }
                                     p_private->p_rowcounter=0;
                                 }
                             }
@@ -1035,9 +1039,11 @@ bool hk_report::save_report(const hk_string& n,bool ask)
 
     reset_has_changed();
     if (name().size()==0) if (!ask_name())return false;
+    hk_database::enum_storage storage=hk_database::st_overwrite;
+    if (p_private->p_loadreportname==name()) storage=hk_database::st_storechanges;
      if (database()->storagemode(ft_report)==hk_database::local)
     {
-      ofstream* p_save=database()->savestream(name(),ft_report,ask);
+      ofstream* p_save=database()->savestream(storage,name(),ft_report,ask,true,false);
       if (p_save==NULL) return false;
       savedata(*p_save);
       p_save->close();
@@ -1045,21 +1051,18 @@ bool hk_report::save_report(const hk_string& n,bool ask)
     }
     else
     { // store central
-#ifdef HAVE_SSTREAM
-       stringstream* p_save;
-#else
-       strstream*  p_save;
-#endif
+       STRINGSTREAM*  p_save;
       p_save=database()->savestringstream(ft_report);
       if (p_save==NULL) return false;
       savedata(*p_save);
       hk_string s=p_save->str();
-      database()->save(s,name(),ft_report,ask);
+      database()->save(storage,s,name(),ft_report,ask);
       delete p_save;
     }
+    p_private->p_loadreportname=name();
     reset_has_changed();
     return true;
-    
+
 }
 
 
@@ -1200,7 +1203,7 @@ bool hk_report::load_report(const hk_string& n)
     hk_dsmodevisible::set_mode(designmode);
     if (name().size()==0) if (!ask_name())return false;
     hk_string res= database()->load(name(),ft_report);
-
+    p_private->p_loadreportname=n;
     if (res.size()==0)
     {
         show_warningmessage(hk_translate("No such report!"));
@@ -2623,3 +2626,34 @@ void hk_report::widget_specific_batch_previous(void)
 
 }
 
+hk_string hk_report::filelocale(void)
+{
+  return p_private->p_locale;
+}
+
+hk_string hk_report::filecharset(void)
+{
+  return p_private->p_charset;
+
+}
+
+void hk_report::set_filelocale(const hk_string& l)
+{
+p_private->p_locale=l;
+}
+
+void hk_report::set_filecharset(const hk_string& l)
+{
+p_private->p_charset=l;
+}
+
+void hk_report::set_progressdialog(progress_dialogtype* progressdialog)
+{
+p_private->p_progressdialog=progressdialog;
+}
+
+progress_dialogtype* hk_report::progressdialog(void)const
+{
+  return p_private->p_progressdialog;
+}
+
