@@ -1,5 +1,5 @@
 // ****************************************************************************
-// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>  
+// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>
 // This file is part of the hk_classes library.
 // This file may be distributed and/or modified under the terms of the
 // GNU Library Public License version 2 as published by the Free Software
@@ -21,6 +21,7 @@
 using namespace std;
 
 class hk_reportsection;
+class hk_reportsectionpair;
 class hk_reportdata;
 class hk_report;
 class hk_column;
@@ -32,7 +33,7 @@ class hk_reportprivate;
  * CVS, HTML, XML, Postscript etc. It has all functions you need to create reports in any kind and any format.
  * See also the convenience class @ref hk_reportcvs.
  * @short basic class to create reports
- *@version $Revision: 1.55 $
+ *@version $Revision: 1.57 $
  * @author Horst Knorr (hk_classes@knoda.org)
 
  *
@@ -427,8 +428,14 @@ hk_string on_new_page_action(void);
 void stop_execution(void);
 bool execution_stopped(void);
 
+        hk_string filelocale(void);
+        hk_string filecharset(void);
+        void set_filelocale(const hk_string&);
+        void set_filecharset(const hk_string&);
 
     protected:
+        void set_progressdialog(progress_dialogtype* progressdialog);
+        progress_dialogtype* progressdialog(void)const;
         void   remove_section(hk_reportsection*);
         virtual hk_string replace(const hk_string& where);
         virtual bool datasource_disable(void);
