@@ -1,5 +1,5 @@
 // ****************************************************************************
-// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>  
+// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>
 // This file is part of the hk_classes library.
 // This file may be distributed and/or modified under the terms of the
 // GNU Library Public License version 2 as published by the Free Software
@@ -8,15 +8,18 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 // ****************************************************************************
-//$Revision: 1.56 $
+//$Revision: 1.63 $
 #include "hk_reportdata.h"
 #include "hk_reportsection.h"
 #include "hk_report.h"
 #include <math.h>
+#include <locale.h>
+#include <nl_types.h>
+#include <langinfo.h>
 #include "hk_reportutils.h"
 #include "hk_column.h"
 #include "hk_datasource.h"
-
+#include <langinfo.h>
 
 class hk_reportdatamodeprivate
 {
@@ -111,6 +114,7 @@ hk_reportdata::hk_reportdata(hk_reportsection* s):hk_dsdatavisible(s->report())
 
     if (p_datareplacefunctions.size()==0)
     {
+        add_datareplacefunctiontype("FILECHARACTERSET",&encodefilecharsetfunction);
         add_datareplacefunctiontype("None",NULL);
     }
     set_replacefunction("None",false);
@@ -168,10 +172,11 @@ hk_string hk_reportdata::replace(const hk_string& where)
 #ifdef HK_DEBUG
     hkdebug("hk_reportdata::replace");
 #endif
-    p_localbuffer=hk_dsdatavisible::replace(where);
+    p_localbuffer=smallstringconversion(hk_dsdatavisible::replace(where),nl_langinfo(CODESET),p_report?p_report->fontencodingstring():"");
     if (!p_report->datasource()) return "";
     list<hk_column*>* cols=p_report->datasource()->columns();
     if (!cols) return "";
+
     list<hk_column*>::iterator it=cols->begin();
     while(it!=cols->end())
     {
@@ -264,22 +269,26 @@ hk_string hk_reportdata::replace(const hk_string& where)
             hk_string name;
             hk_string display;
 
-            if (is_numerictype(column()))
+            if (!column()->is_nullvalue())
             {
+             if (is_numerictype(column()))
+             {
                 double n=localestring2double(column()->asstring(true));
-                value=format_number(n,use_numberseparator(),commadigits()<0?(is_integertype(column())?0:-1):commadigits());
+                value=format_number(n,use_numberseparator(),commadigits()<0?(is_integertype(column())?0:-1):commadigits(),p_report->filelocale());
 //                value=format_number(n,use_numberseparator(),is_integertype(column())?(commadigits()<0)?0:commadigits():(commadigits()<0)?-1:commadigits());
-            }
-            else   value=column()->asstring(true);
-            name=columnname();
-            display=p_viewdata->p_displayname;
-
-            if (r!=NULL)
-            {
+             }
+             else   value=column()->asstring(true);
+	    }
+             name=columnname();
+             display=p_viewdata->p_displayname;
+             
+             if (r!=NULL)
+              {
                 value=r(value,p_report);
                 name=r(name,p_report);
                 display=r(display,p_report);
-            }
+              }
+            
             p_localbuffer=replace_all("%VALUE%",p_localbuffer,value);
             p_localbuffer=replace_all("%FIELDNAME%",p_localbuffer,name);
             p_localbuffer=replace_all("%DISPLAYNAME%",p_localbuffer,display);
@@ -309,7 +318,7 @@ hk_string hk_reportdata::actual_string(void)
     r=p_viewdata->p_beforedata+p_viewdata->p_data+p_viewdata->p_afterdata;
     r=replace(r);
     b=r;
-    if (p_private->p_replacefunction!=NULL) r=p_private->p_replacefunction(this,r);
+    if (p_private->p_replacefunction!=NULL) b=p_private->p_replacefunction(this,r);
     if (p_private->p_datacountfunction!=NULL)set_counts_as(p_private->p_datacountfunction(this));
     p_report->rowcount(counts_as());
     r=b;
@@ -916,9 +925,9 @@ p_setwidgetcoordinates=false;
 }
 
 
-void hk_reportdata::presentationmode_changed(void)
+bool hk_reportdata::presentationmode_changed(void)
 {
-hk_dsdatavisible::presentationmode_changed();
+bool r=hk_dsdatavisible::presentationmode_changed();
     if (p_report->mode()==hk_presentation::viewmode)
     {
         p_viewdata->p_data=p_designdata->p_data;
@@ -932,6 +941,7 @@ hk_dsdatavisible::presentationmode_changed();
 	p_viewdata->p_luro=p_designdata->p_luro;
 	p_viewdata->p_loru=p_designdata->p_loru;
    }
+ return r;
 }
 
 
