@@ -1,5 +1,5 @@
 // ****************************************************************************
-// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>  
+// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>
 // This file is part of the hk_classes library.
 // This file may be distributed and/or modified under the terms of the
 // GNU Library Public License version 2 as published by the Free Software
@@ -8,7 +8,7 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 // ****************************************************************************
-//$Revision: 1.61 $
+//$Revision: 1.62 $
 #include "hk_reportsection.h"
 #include "hk_reportsectionpair.h"
 #include "hk_report.h"
@@ -577,9 +577,9 @@ bool hk_reportsection::print_subreport(void)
 #ifdef HK_DEBUG
     hkdebug("hk_reportsection::print_subreport");
 #endif
-    if (p_subreport!=NULL) 
+    if (p_subreport!=NULL)
        return p_subreport->execute();
-   return true;   
+   return true;
 }
 
 
@@ -629,7 +629,7 @@ void    hk_reportsection::set_subreport(const hk_string& subreport,bool before_d
     {
        subds->clear_depending_fields();
        subds->set_depending_on(NULL,false);
-    }        
+    }
     delete p_subreport;
     p_subreport=NULL;
 
@@ -1037,7 +1037,7 @@ unsigned int    hk_reportsection::horizontal2relativ(unsigned int h)
       return (unsigned int) ((h*teiler) / (p_report->designwidth()-p_report->relativ2horizontal(p_report->border_left()+p_report->border_right()))+0.5);
     else
       return (unsigned int) ((h*teiler) / (p_report->designwidth()- p_report->border_left()-p_report->border_right())+0.5);
-    
+
 }
 
 
@@ -1244,15 +1244,16 @@ return NULL;
 }
 
 
-void	hk_reportsection::presentationmode_changed(void)
+bool	hk_reportsection::presentationmode_changed(void)
 {
+   bool r=true;
    vector<hk_reportdata*>::iterator vit=p_data.begin();
-    while (vit!=p_data.end())
+    while (r&&vit!=p_data.end())
     {
-        (*vit)->presentationmode_changed();
+        if (!(*vit)->presentationmode_changed())r=false;
         vit++;
     }
-
+  return r;
 }
 
 hk_string hk_reportsection::sectioncountfunctionstring(void)
