@@ -1,5 +1,5 @@
 // ****************************************************************************
-// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>  
+// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>
 // This file is part of the hk_classes library.
 // This file may be distributed and/or modified under the terms of the
 // GNU Library Public License version 2 as published by the Free Software
@@ -8,7 +8,7 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 // ****************************************************************************
-//$Revision: 1.59 $
+//$Revision: 1.62 $
 #include "hk_reportutils.h"
 #include "hk_reportsectionpair.h"
 #include "hk_datasource.h"
@@ -970,10 +970,14 @@ void hk_reportcsv::set_multiplefiles(bool m)
 }
 
 
-void    hk_reportcsv::set_dataformat(const hk_string& delimiter,const hk_string& between)
+void    hk_reportcsv::set_filedefinition(const hk_string& textdelimiter,const hk_string& betweenfields,const hk_string& rowdelimiter,const hk_string& datetimeformat,const hk_string& dateformat,const hk_string& timeformat, const hk_string& charset,const hk_string& locale)
 {
-    p_delimiter=delimiter;
-    p_between=between;
+    p_delimiter=textdelimiter;
+    p_between=betweenfields;
+    set_datetimeformat(datetimeformat,dateformat,timeformat);
+    set_filelocale(locale);
+    set_filecharset(charset);
+
     configure_page();
 
 }
@@ -1023,8 +1027,8 @@ void hk_reportcsv::configure_page(void)
 
 
 void    hk_reportcsv::widget_specific_before_execute(void)
-{
-    setlocale(LC_NUMERIC,"C");
+{ 
+    setlocale(LC_NUMERIC,filelocale().c_str());
     datasection()->set_automaticcreatedatafunction(&automatic_create_csvfields);
 }
 
@@ -1035,33 +1039,40 @@ void    hk_reportcsv::widget_specific_after_execute(void)
 }
 
 
-void set_cvs(hk_report* e,bool with_header,const hk_string& col_delimiter, const hk_string& betweendata)
+// void set_cvs(hk_report* e,bool with_header,const hk_string& col_delimiter, const hk_string& betweendata)
+// {
+//     if (e==NULL)return;
+//     hk_string delimiter=col_delimiter;
+//     hk_string between=betweendata;
+//     if (with_header)
+//     {
+//
+//         hk_reportsectionpair* header=e->new_sectionpair();
+//         hk_string h=delimiter+"%FIELDNAME%";
+//         h+=delimiter;
+//         header->headersection()->set_default_reportdata(h);
+//         header->headersection()->set_betweendata(between);
+//         header->headersection()->set_sectionend("\n");
+//         header->headersection()->set_unique();
+//
+//     }
+//     hk_reportsection* section=e->datasection();
+//     hk_string h=delimiter+"%VALUE%";
+//     h+=delimiter;
+//     section->set_default_reportdata(h);
+//     section->set_betweendata(between);
+//     section->set_sectionend("\n");
+//
+// }
+//
+
+
+hk_string encodefilecharsetfunction(hk_reportdata* rd,const hk_string& value)
 {
-    if (e==NULL)return;
-    hk_string delimiter=col_delimiter;
-    hk_string between=betweendata;
-    if (with_header)
-    {
-
-        hk_reportsectionpair* header=e->new_sectionpair();
-        hk_string h=delimiter+"%FIELDNAME%";
-        h+=delimiter;
-        header->headersection()->set_default_reportdata(h);
-        header->headersection()->set_betweendata(between);
-        header->headersection()->set_sectionend("\n");
-        header->headersection()->set_unique();
-
-    }
-    hk_reportsection* section=e->datasection();
-    hk_string h=delimiter+"%VALUE%";
-    h+=delimiter;
-    section->set_default_reportdata(h);
-    section->set_betweendata(between);
-    section->set_sectionend("\n");
-
+if (!rd) return value;
+return smallstringconversion(value,"",rd->report()->filecharset());
 }
 
-
 void automatic_create_csvfields(hk_reportsection* s)
 {
     if (!s) return;
@@ -1075,6 +1086,8 @@ void automatic_create_csvfields(hk_reportsection* s)
         while (cit!=cols->end())
         {
             hk_reportdata* dat=s->new_data();
+            dat->set_replacefunction("FILECHARACTERSET",false);
+            dat->set_datetimeformat(rcsv->datetimeformat(),rcsv->dateformat(),rcsv->timeformat());
             dat->set_columnname((*cit)->name());
             dat->set_data(s->default_reportdata());
             if ((*cit)->columntype()==hk_column::textcolumn || (*cit)->columntype()==hk_column::memocolumn)
