@@ -1,5 +1,5 @@
 // ****************************************************************************
-// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>  
+// copyright (c) 2000-2005 Horst Knorr <hk_classes@knoda.org>
 // This file is part of the hk_classes library.
 // This file may be distributed and/or modified under the terms of the
 // GNU Library Public License version 2 as published by the Free Software
@@ -8,23 +8,31 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 // ****************************************************************************
-//$Revision: 1.79 $
+//$Revision: 1.87 $
 #include "hk_visible.h"
 #include "hk_presentation.h"
 #include "hk_form.h"
 #include "hk_report.h"
 #include "hk_datetime.h"
 #include "hk_interpreter.h"
+#include "hk_tabvisible.h"
+#include "hk_label.h"
 
 class hk_visiblemodeprivate
 {
   public:
+hk_visiblemodeprivate()
+{
+p_buddylabel=-1;
+p_enabled=true;
+}
         unsigned int    p_x,p_y,p_width,p_height;
         hk_string p_datetimeformat;
         hk_string p_dateformat;
         hk_string p_timeformat;
         unsigned long int p_counts_as;
         bool p_load_presentationnumber;
+        bool p_enabled;
         hk_visible::alignmenttype    p_align;
         hk_colour   p_foregroundcolour;
         hk_colour   p_backgroundcolour;
@@ -33,6 +41,7 @@ class hk_visiblemodeprivate
         hk_string p_clickaction,p_doubleclickaction, p_onopenaction,p_oncloseaction;
 	hk_string p_ongetfocusaction,p_onloosefocusaction,p_onkeyaction;
 	hk_string p_tooltip;
+	long int p_buddylabel;
 	bool  operator=(const hk_visiblemodeprivate* v)
      {
 		if (!v) return false;
@@ -58,6 +67,8 @@ class hk_visiblemodeprivate
 		p_onloosefocusaction=v->p_onloosefocusaction;
 		p_onkeyaction=v->p_onkeyaction;
 		p_tooltip=v->p_tooltip;
+		p_buddylabel=v->p_buddylabel;
+		p_enabled=v->p_enabled;
         return true;
      }
 
@@ -68,11 +79,14 @@ class hk_visibleprivate
 public:
     hk_visibleprivate()
     {
-     ;
+      p_tabvisible=NULL;
+      p_movebuddylabel=true;
     }
 	hk_string p_identifier;
         bool        p_neverregisterchange;
 	hk_key p_key;
+	hk_tabvisible* p_tabvisible;
+	bool p_movebuddylabel;
 
 };
 
@@ -191,8 +205,14 @@ void hk_visible::set_size(unsigned int x, unsigned int y,unsigned int w, unsigne
         ppw=p_presentation->relativ2horizontal(ppw);
 
     }
-    
-    
+
+  hk_label* l=NULL;
+   hk_form* f=dynamic_cast<hk_form*>(p_presentation);
+    if (f!=NULL && buddylabel()>-1)
+    {
+       l=dynamic_cast<hk_label*>(f->get_visible(buddylabel()));
+    }
+
     if (p_setwidgetcoordinates)
     {
 #ifdef HK_DEBUG
@@ -205,6 +225,23 @@ void hk_visible::set_size(unsigned int x, unsigned int y,unsigned int w, unsigne
         p_designdata->p_width=usew;
         p_designdata->p_height=useh;
     }
+     if (l && p_private->p_movebuddylabel)
+     {
+       int dx=usex-p_viewdata->p_x;
+       int dy=usey-p_viewdata->p_y;
+       int nx=l->x()+dx;
+       if (nx<0)nx=0;
+       int ny=l->y()+dy;
+       if (ny<0)ny=0;
+       if (p_presentation)
+       {
+          if (nx+l->width()>p_presentation->designwidth()) nx=(p_presentation->designwidth()-l->width());
+          if (ny+l->height()>p_presentation->designheight()) ny=(p_presentation->designheight()-l->height());
+       }
+       l->set_position(nx,ny);
+
+     }
+
 	p_viewdata->p_x=usex;
         p_viewdata->p_y=usey;
         p_viewdata->p_width=usew;
@@ -371,6 +408,29 @@ if (p_presentation && p_presentation->mode()==hk_presentation::viewmode)
 }
 
 
+void hk_visible::set_enabled(bool e, bool registerchange,bool force_setting)
+{
+#ifdef HK_DEBUG
+    hkdebug("hk_visible::set_label(hk_string)");
+#endif
+    if (allow_datachanging(force_setting))
+    {
+    p_designdata->p_enabled=e;
+    }
+    p_viewdata->p_enabled=e;
+    has_changed(registerchange);
+    widget_specific_enabled_changed();
+
+}
+
+bool hk_visible::is_enabled(void)
+{
+if (p_presentation && p_presentation->mode()==hk_presentation::viewmode)
+    return p_viewdata->p_enabled;
+return p_designdata->p_enabled;
+
+}
+
 bool hk_visible::set_identifier(const hk_string& i,bool registerchange,enum_interaction a)
 {
 if (i==p_private->p_identifier)return true;
@@ -378,7 +438,7 @@ if(i.size()==0) return false;
 hk_form* f=dynamic_cast<hk_form*> (p_presentation);
 if (f &&f->get_visible(i))
   {
-    if (a==interactive)show_warningmessage(hk_translate("Identifier already in use"));
+    if (a==interactive)show_warningmessage(replace_all("%1",hk_translate("Identifier '%1' already in use"),i));
     return false;
   }
   else
@@ -386,14 +446,13 @@ if (f &&f->get_visible(i))
   hk_report* f=dynamic_cast<hk_report*> (p_presentation);
   if (f &&f->get_reportdatavisible(i))
   {
-    if (a==interactive)show_warningmessage(hk_translate("Identifier already in use"));
+    if (a==interactive)show_warningmessage(replace_all("%1",hk_translate("Identifier '%1' already in use"),i));
     return false;
   }
 
 
   }
 
-
 p_private->p_identifier=i;
 has_changed(registerchange);
 return true;
@@ -427,6 +486,9 @@ void hk_visible::savedata(ostream& s )
         case report       :  v="REPORT";break;
         case reportsection    :  v="REPORTSECTION";break;
         case reportdata   :  v="REPORTDATA";break;
+        case image   :  v="IMAGE";break;
+        case date   :  v="DATE";break;
+        case tabvisible   :  v="TABVISIBLE";break;
         default               : v="UNKNOWN";
     } ;
     set_tagvalue(s,"VISIBLETYPE",v);
@@ -440,6 +502,8 @@ void hk_visible::savedata(ostream& s )
     set_tagvalue(s,"VUPN",p_vupn);
     set_tagvalue(s,"COUNTS_AS",p_designdata->p_counts_as);
     set_tagvalue(s,"TOOLTIP",p_designdata->p_tooltip);
+    set_tagvalue(s,"BUDDYLABEL",p_designdata->p_buddylabel);
+    set_tagvalue(s,"ISENABLED",p_designdata->p_enabled);
 
     hk_string coltag="FOREGROUNDCOLOUR";
     start_mastertag(s,coltag);
@@ -489,6 +553,8 @@ void hk_visible::loaddata(const hk_string& definition)
     get_tagvalue(definition,"Y", p_designdata->p_y);
     get_tagvalue(definition,"WIDTH", p_designdata->p_width);
     get_tagvalue(definition,"HEIGHT", p_designdata->p_height);
+    get_tagvalue(definition,"BUDDYLABEL",p_designdata->p_buddylabel);
+    get_tagvalue(definition,"ISENABLED",p_designdata->p_enabled);
     if (get_tagvalue(definition,"HK_FONT",v))
     {
      p_designdata->p_font.loaddata(v);
@@ -513,7 +579,7 @@ void hk_visible::loaddata(const hk_string& definition)
     if (align=="RIGHT") p_designdata->p_align=alignright;
     else if (align=="CENTER")p_designdata->p_align=aligncenter; else p_designdata->p_align=alignleft;
     set_alignment(p_designdata->p_align,false);
-    
+
     if (get_tagvalue(definition,"TOOLTIP",p_designdata->p_tooltip))
        set_tooltip(p_designdata->p_tooltip,false);
 
@@ -595,6 +661,11 @@ void    hk_visible::widget_specific_label_changed(void)
 
 }
 
+void    hk_visible::widget_specific_enabled_changed(void)
+{
+
+}
+
 
 void hk_visible::set_neverregisterchange(bool n)
 {
@@ -833,7 +904,7 @@ void   hk_visible::set_foregroundcolour(const hk_colour& colour,bool registercha
         p_designdata->p_foregroundcolour=colour;
         has_changed(registerchange);
     }
-    
+
     widget_specific_foregroundcolour_changed(oldcolour);
 
 }
@@ -960,9 +1031,10 @@ bool        hk_visible::allow_datachanging(bool force_setting)
 }
 
 
-void hk_visible::presentationmode_changed(void)
+bool hk_visible::presentationmode_changed(void)
 {
-   if (!p_presentation) return;
+   if (!p_presentation) return true;
+   bool result=true;
    switch (p_presentation->mode())
    {
      	case hk_presentation::designmode : ;
@@ -972,15 +1044,16 @@ void hk_visible::presentationmode_changed(void)
 							widget_specific_font_changed();
 							widget_specific_alignment();
     							set_size(p_designdata->p_x,p_designdata->p_y,p_designdata->p_width,p_designdata->p_height,false);
-							action_on_close();
+							if (!action_on_close()) result=false;
+							show_widget();
      							break;
 	case hk_presentation::viewmode:	;
 							*p_viewdata=*p_designdata;
-							action_on_open();
+							if (!action_on_open()) result=false;
 							break;
    }
 
-
+return result;
 }
 
 
@@ -1115,7 +1188,7 @@ void hk_visible::set_tooltip(const hk_string& t,bool registerchange,bool force_s
 }
 
 
-hk_string hk_visible::tooltip() 
+hk_string hk_visible::tooltip()
 {
 if (p_presentation && p_presentation->mode()==hk_presentation::viewmode)
     return p_viewdata->p_tooltip;
@@ -1228,14 +1301,65 @@ hk_key* hk_visible::key(void) const
 {
 //cerr <<"hk_key hk_visible::key"<<endl;
   return &p_private->p_key;
-}  
+}
 
 
+void hk_visible::show_widget(void)
+{
 
+}
+void hk_visible::hide_widget(void)
+{
 
+}
 
+void hk_visible::close_widget(void)
+{
 
+}
 
 
+ bool hk_visible::in_tabvisible(void) const
+ {
+   return (p_private->p_tabvisible!=NULL);
+ }
+
+void hk_visible::set_tabvisible(hk_tabvisible* t)
+{
+  p_private->p_tabvisible=t;
+}
+
+
+void hk_visible::set_buddylabel(long int labelid,bool registerchange,bool force_setting)
+{
+    if (allow_datachanging(force_setting))
+    {
+    p_designdata->p_buddylabel=labelid;
+    }
+    p_viewdata->p_buddylabel=labelid;
+    has_changed(registerchange);
+
+}
+
+long int hk_visible::buddylabel(void) const
+{
+if (p_presentation && p_presentation->mode()==hk_presentation::viewmode)
+    return p_viewdata->p_buddylabel;
+
+    return p_designdata->p_buddylabel;
+
+}
+
+
+void hk_visible::set_movebuddylabel(bool m)
+{
+  p_private->p_movebuddylabel=m;
+}
+
+bool hk_visible::move_buddylabel(void) const
+{
+  return p_private->p_movebuddylabel;
+}
+
 
 
