@@ -8,7 +8,7 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 // ****************************************************************************
-//$Revision: 1.15 $
+//$Revision: 1.18 $
 #include "hk_odbctable.h"
 #include "hk_odbcdatabase.h"
 #include "hk_odbcconnection.h"
@@ -365,14 +365,20 @@ list<hk_datasource::indexclass>* hk_odbctable::driver_specific_indices(void)
     p_indices.erase(p_indices.begin(),p_indices.end());
     SQLHSTMT  result;
     SQLRETURN ret= SQLAllocHandle(SQL_HANDLE_STMT,p_odbcdatabase->connection()->connectionhandle(),&result);
+    if (ret == SQL_ERROR || ret == SQL_INVALID_HANDLE) return &p_indices;
     int unique;
     SQLLEN indicator;
     const int buffersize=100;
     char* buffer=new char[buffersize];
-    ret=SQLStatistics( result, 0, 0, 0, 0, (SQLCHAR*)name().c_str(), SQL_NTS, 0, 0 );
+    SQLCHAR* namebuffer=new SQLCHAR[name().size()+1];
+    strncpy((char*)namebuffer,name().c_str(),name().size()+1);
+     cerr <<"BEFORE SQLStatistics '"<<name()<<"' buffer ='"<<namebuffer<<"'"<<endl;
+    ret=SQLStatistics( result, 0, 0, 0, 0, namebuffer, SQL_NTS, 0, 0 );
+    cerr <<"AFTER SQLStatistics"<<endl;
+    delete[] namebuffer;
     hk_string currentindexname;
     indexclass* index=NULL;
-
+     cerr<<"Start FETCHING"<<endl;
     if (ret==SQL_SUCCESS)
     {
 // now fetch the index definitions
@@ -403,7 +409,7 @@ list<hk_datasource::indexclass>* hk_odbctable::driver_specific_indices(void)
         }
 
     }
-
+   cerr <<"BEFORE GET PRIMARY"<<endl;
     //now get Primary keys
         SQLFreeHandle(SQL_HANDLE_STMT,result);
     SQLAllocHandle(SQL_HANDLE_STMT,p_odbcdatabase->connection()->connectionhandle(),&result);
@@ -433,7 +439,7 @@ cerr <<endl<<"PRIMARYKEYS werden ermittelt"<<endl<<endl;
             ret = SQLFetch( result );
         }                                         //end while
 
-cerr <<"ENDE Primarykeys"<<endl;
+cerr <<"END Primarykeys"<<endl;
     }else cerr <<"Fehler bei SQLPrimarykeys!!!"<<endl<<endl;
 
     delete[] buffer;
